#
# Copyright (c) 2013 by DarkDefender <darkdefende (a) gmail.com>
# File bugs here: https://github.com/DarkDefender/weetweet

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import sys
import ast
import re
import os
import tempfile
import time
import calendar
import socket
import getpass

# This twitter plugin can be extended even more. Just look at the twitter api
# doc here: https://dev.twitter.com/docs/api/1.1

# Thanks to ttytter for interface ideas!
# http://www.floodgap.com/software/ttytter/

# I've borrowed some ideas, functions for ainmosni and his twitter plugin
# https://github.com/ainmosni/weetwit

weechat_call = True
import_ok = True
required_twitter_version = "1.14.1"

try:
    import weechat
    #We need to import sys to see which python version weechat runs (2 or 3)
    import sys
    is_py3 = sys.version_info >= (3, 0)
except:
    #import html parser so we can convert html strings to plain text
    try:
        import html.parser
    except:
        print("You need to have python3 installed to run this script!")
        exit(0)
    weechat_call = False

try:
    #Import python twitter lib
    from twitter import *
    from twitter.stream import Timeout, HeartbeatTimeout, Hangup
except:
    import_ok = False

try:
    #Import for version checking
    from pkg_resources import parse_version, get_distribution
    version = get_distribution("twitter").version
    if parse_version(required_twitter_version) > parse_version(version):
        import_ok = False
except:
    if weechat_call:
        weechat.prnt("", "You need to have pkg_resources installed for version checking")
    else:
        print("You need to have pkg_resources installed for version checking")

# These two keys is what identifies this twitter client as "weechat twitter"
# If you want to change it you can register your own keys at:
# https://dev.twitter.com/apps/new

CONSUMER_SECRET = 'ivx3oxxkSOAOofRuhmGXQK4nkLFNXD94wbJiRUBhN1g'
CONSUMER_KEY = 'NVkYe8DAeaw6YRcjw662ZQ'

script_options = {
    "oauth_token" : "",
    "oauth_secret" : "",
    "auth_complete" : False,
    "screen_name" : "",
    "last_id" : "",
    "print_id" : True,
    "alt_rt_style" : False,
    "home_replies" : False,
    "tweet_nicks" : True,
}

#TODO have a dict for each buffer
tweet_dict = {'cur_index': "a0"}
#Mega command dict
command_dict = dict(user="u",replies="r",view_tweet="v",thread="th",link="l",
        retweet="rt",delete="d",tweet="t",reply="re",new_tweets="new",
        follow_user="follow",unfollow_user="unfollow",following="f",
        followers="fo",about="a",block="b",unblock="ub",
        blocked_users="blocks",favorite="fav",unfavorite="unfav",
        favorites="favs",fav_retweet="fart",rate_limits="limits",home_timeline="home",
        clear_nicks="cnicks",clear_buffer="clear",create_stream="stream",
        restart_home_stream="re_home")
desc_dict = dict(
        user="<user> [<id> <count>|<id>|<count>], Request user timeline, " +
        "if <id> is given it will get tweets older than <id>, " +
        "<count> is how many tweets to get, valid number is 1-200",
        replies="[<id> <count>|<id>|<count>],Get any replies/mentions of you " +
        "if <id> is given it will get tweets older than <id>, " +
        "<count> is how many tweets to get, valid number is 1-200",
        view_tweet="<id>, View/get tweet with <id>",
        thread="<id>, View/get the reply chain of tweets (the thread) where " +
        "<id> is the last tweet in the thread.",
        link="<id>, Get a link to tweet with <id>",
        retweet="<id>, Retweet <id>",
        delete="<id>, Delete tweet <id>. You can only delete your own tweets...",
        tweet="<text>Tweet the text following this command",
        reply="<id> <text>, reply to <id>. You need to have @<username> " +
        "of the user that you reply to in the tweet text. If this is not " +
        "the case this will be treated like a normal tweet instead.",
        new_tweets="Get new tweets from your home_timeline. This is only " +
        "useful if you have disconnected from the home twitter stream",
        follow_user="<user>, Add user to people you follow",
        unfollow_user="<user>, Remove user for people you follow",
        following="[|<id>|<user>|<user> <id>], Show 'friends' of <user> or " +
        "if no user were given show the people you follow. If not all " +
        "followers were printed supply the <id> of the last list to get " +
        "the new batch of nicks",
        followers="[|<id>|<user>|<user> <id>], Who followes <user> or " +
        "if no user were given show your follower. If not all " +
        "followers were printed supply the <id> of the last list to get " +
        "the new batch of nicks",
        about="<user>, Print info about <user>",
        block="<user>, Block <user>",
        unblock="<user>, Unblock <user>",
        blocked_users="Print a list of users you have currently blocked",
        favorite="<id>, Add tweet <id> to you favorites",
        unfavorite="<id>, Remove tweet <id> from yout favorites",
        favorites="[|<user>][<id> <count>|<id>|<count>], Request <user> favs, " +
        "if <user> is not given get your own favs. " +
        "If <id> is given it will get tweets older than <id>, " +
        "<count> is how many tweets to get, valid number is 1-200",
        fav_retweet="<id>, favorites and retweets <id>",
        rate_limits="[|<sub_group>], get the current status of the twitter " +
        "api limits. It prints how much you have left/used. " +
        " if <sub_group> is supplied it will only get/print that sub_group.",
        home_timeline="[<id> <count>|<id>|<count>],Get tweets from you home " +
        "timeline" +
        "if <id> is given it will get tweets older than <id>, " +
        "<count> is how many tweets to get, valid number is 1-200",
        clear_nicks="Clear nicks from the 'Tweet_parse' nick group. "+
        "These nicks are parsed from recived tweets, it can get " +
        "messy at times...",
        clear_buffer="Clear the twitter buffer of text "+
        "same as '/buffer clear'",
        create_stream="Create a twitter stream with the following filter "+
        "options: <users to stream> <keywords>. Note that the user list and the "+
        "keyword list must be comma seperated. IE user1,user2,user3 keyword1,keyword2.\n" +
        "If you only want to stream keywords, write '&' as 'users' input.\n"+
        "NOTE: you can only have one stream at a time because "+
        "twitter will IP ban you if you repeatedly request more than one "+
        "stream.",
        restart_home_stream="Restart the home timeline stream after it has " +
        "shutdown.")

SCRIPT_NAME = "weetweet"
SCRIPT_FILE_PATH = os.path.abspath(__file__)

twit_buf = ""
sock_hooks = {}
proc_hooks = {}
sock_fd_dict = {}
tweet_nicks_group = {}
friends_nicks_group = {}

html_escape_table = {
    '"': "&quot;",
    "'": "&apos;",
    }

def html_escape(text):
    """Produce entities within text."""
    return "".join(html_escape_table.get(c,c) for c in text)

def dict_tweet(tweet_id):
    cur_index = tweet_dict['cur_index']
    if not tweet_id in tweet_dict.values():
        if cur_index == 'z9':
            cur_index = 'a0'

        if cur_index[1] == '9':
            cur_index = chr(ord(cur_index[0]) + 1) + '0'
        else:
            cur_index = cur_index[0] + chr(ord(cur_index[1]) + 1)

        tweet_dict[cur_index] = tweet_id
        tweet_dict['cur_index'] = cur_index
        return cur_index
    else:
        for index, t_id in tweet_dict.items():
            if t_id == tweet_id:
                return index


def read_config():
    for item in script_options:
        script_options[item] = weechat.config_string(weechat.config_get("plugins.var.python."+SCRIPT_NAME+"." + item))
    for item in ["auth_complete","print_id","alt_rt_style","home_replies","tweet_nicks"]:
        #Convert to bool
        script_options[item] = weechat.config_string_to_boolean(script_options[item])
    for item in ["oauth_token","oauth_secret"]:
        #Convert potentially encrypted tokens
        script_options[item] = weechat.string_eval_expression(script_options[item], {}, {}, {})

def config_cb(data, option, value):
    """Callback called when a script option is changed."""
    # for example, read all script options to script variables...
    # ...
    read_config()
    return weechat.WEECHAT_RC_OK

def add_to_nicklist(buf, nick, group=""):
    """Add nick to the nicklist."""
    if group == "":
        group = friends_nicks_group[buf]
    weechat.nicklist_add_nick(buf, group, nick, 'bar_fg', '', '', 1)

def remove_from_nicklist(buf, nick, group=""):
    """Remove nick from the nicklist."""
    if group == "":
        group = friends_nicks_group[buf]
    nick_ptr = weechat.nicklist_search_nick(buf, group, nick)
    weechat.nicklist_remove_nick(buf, nick_ptr)

def parse_for_nicks(text,buffer):
    #Parse text for twitter nicks and add them to nicklist
    regex = re.compile(r'@([A-Za-z0-9_]+)')
    for nick in regex.findall(text):
        add_to_nicklist(buffer,nick,tweet_nicks_group[buffer])

def print_tweet_data(buffer,tweets,data):

    for message in tweets:
        nick = message[1]
        text = message[3]
        reply_id = ""
        if script_options['tweet_nicks']:
            parse_for_nicks(text,buffer)
            add_to_nicklist(buffer,nick,tweet_nicks_group[buffer])

        if script_options['print_id']:
            t_id = weechat.color('reset') + ' ' + dict_tweet(message[2])
        else:
            t_id = ''

        if len(message) == 5:
            #This is a reply to a tweet
            arrow_col = weechat.color('chat_prefix_suffix')
            reset_col = weechat.color('reset')
            reply_id = arrow_col +  "<" + reset_col + dict_tweet(message[4]) + arrow_col + "> " + reset_col
            temp_text = text
            text = reply_id
            reply_id = temp_text

        weechat.prnt_date_tags(buffer, message[0], "notify_message",
                "%s%s\t%s%s" % (nick, t_id, text,reply_id))
    if data == "id":
        try:
            if script_options['last_id'] < tweets[-1][2]:
                script_options['last_id'] = tweets[-1][2]
                # Save last id
                weechat.config_set_plugin("last_id",script_options["last_id"])
        except:
            pass

def trim_tweet_data(tweet_data, screen_name, alt_rt_style, is_py3):
    # Because of the huge amount of data, we need to cut down on most of it because we only really want
    # a small subset of it. This also prevents the output buffer from overflowing when fetching many tweets
    # at once.

    h = html.parser.HTMLParser()

    output = []
    for message in tweet_data:
        if message.get('retweeted_status'):
            if alt_rt_style:
                if message['user']['screen_name'] == screen_name:
                    #escape highlighting
                    message['user']['screen_name'] = "<you>"
                message['text'] = message['retweeted_status']['text'] + " (retweeted by " + message['user']['screen_name'] + ")"
                message['user'] = message['retweeted_status']['user']
            else:
                message['text'] = "RT @{}: {}".format(message['retweeted_status']['user']['screen_name'],
                                                      message['retweeted_status']['text'])
        mes_list = [calendar.timegm(time.strptime(message['created_at'],'%a %b %d %H:%M:%S +0000 %Y')),
            message['user']['screen_name'],
            message['id_str']]
        if is_py3:
            mes_list.append(h.unescape(message['text']))
        else:
            #convert text to bytes so python2 can read it correctly
            mes_list.append(h.unescape(message['text']).encode('utf-8'))
        if message["in_reply_to_status_id_str"] != None:
            mes_list.append(message["in_reply_to_status_id_str"])

        output.append(mes_list)

    output.reverse()
    return output

def stream_message(buffer,tweet):
    if 'delete' in tweet:
        #Colorize the tweet id
        arrow_col = weechat.color('chat_prefix_suffix')
        reset_col = weechat.color('reset')
        dict_id = dict_tweet(tweet['delete']['status']['id_str'])
        id_str = arrow_col +  "<" + reset_col + dict_id + arrow_col + "> " + reset_col
        weechat.prnt(buffer, "%s%s" % (weechat.prefix("network"),
        "Got request to delete: " + id_str))
    elif 'event' in tweet:
        event_str = tweet['event']

        if event_str[-1] == "e":
            event_str += "d"
        elif event_str[-1] != "d":
            event_str += "ed"

        extra_str = ""

        if 'target_object' in tweet:
            if 'id_str' in tweet['target_object']:
                arrow_col = weechat.color('chat_prefix_suffix')
                reset_col = weechat.color('reset')
                dict_id = dict_tweet(tweet['target_object']['id_str'])
                extra_str = "'s tweet " + arrow_col +  "<" + reset_col + dict_id + arrow_col + "> " + reset_col

        #TODO make the event printing better
        weechat.prnt_date_tags(buffer, 0, "no_highlight", "%s%s" % (weechat.prefix("network"),
            tweet['source']['screen_name'] + " " + event_str + " " + tweet['target']['screen_name'] + extra_str))
    elif 'friends' in tweet:
        #This should be the initital message you get listing friend ids when connecting to the twitter stream
        weechat.prnt(buffer, "%s%s" % (weechat.prefix("network"), "Connected to twitter streaming API."))
        #Get new tweets since the last update (we might have missed some if this is after a reconnect)
        #Get latest tweets from timeline
        my_command_cb("silent", buffer, "new")
    else:
        weechat.prnt(buffer, "%s%s" % (weechat.prefix("network"),
        "recv stream data: " + str(tweet)))

def twitter_stream_cb(buffer,fd):

    #accept connection
    server = sock_fd_dict[ sock_fd_dict[str(fd)] ]
    conn, addr = server.accept()
    tweet = ""
    data = True
    while data:
        try:
            data = conn.recv(1024).decode('utf-8')
            tweet += data
        except:
            break

    try:
        tweet = ast.literal_eval(tweet)
    except:
        weechat.prnt(buffer, "Error recv stream message")
        return weechat.WEECHAT_RC_OK
    #Is this a text message (normal tweet)?
    if isinstance(tweet,list):
        if buffer == twit_buf:
            #Update last recv id
            print_tweet_data(buffer,tweet,"id")
        else:
            print_tweet_data(buffer,tweet,"")
    elif tweet == "options":
        #We need to send over the stream options

        options = dict(screen_name = script_options['screen_name'],
                name = sock_fd_dict[str(fd)],
                alt_rt_style = int(script_options['alt_rt_style']),
                home_replies = int(script_options['home_replies']),
                token = script_options["oauth_token"],
                secret = script_options["oauth_secret"],
                is_py3 = is_py3)

        if is_py3:
            conn.sendall(bytes(str(options),"utf-8"))
        else:
            conn.sendall(bytes(str(options)))
    else:
        #https://dev.twitter.com/docs/streaming-apis/messages
        #TODO handle stream events
        stream_message(buffer,tweet)

    conn.close()
    return weechat.WEECHAT_RC_OK

def twitter_stream(cmd_args):
    if len(cmd_args) < 3:
        return "Invalid stream command"
    if not os.path.exists(cmd_args[2]):
        return "The socket file doesn't exist! " + cmd_args[2]

    def connect():
        client = socket.socket( socket.AF_UNIX, socket.SOCK_STREAM )
        client.connect(cmd_args[2])
        #Don't block by default, timeout if no data is present
        client.setblocking(False)
        return client

    client = connect()
    client.settimeout(3)
    client.sendall(bytes('"options"',"utf-8"))
    client.shutdown(socket.SHUT_WR)

    data = True
    options = ""

    while data:
        try:
            data = client.recv(1024).decode('utf-8')
            options += data
        except:
            break

    try:
        option_dict = ast.literal_eval(options)
    except:
        return "Did not manage to get startup arguments to stream"

    client.close()

    oauth_token = option_dict['token']
    oauth_secret= option_dict['secret']
    home_replies = option_dict['home_replies']
    alt_rt_style = option_dict['alt_rt_style']
    screen_name = option_dict['screen_name']
    name = option_dict['name']
    is_py3 = option_dict['is_py3']

    if len(cmd_args) >= 4:
        stream_args = cmd_args[3:]
    else:
        stream_args = ""

    # These arguments are optional. But the current code only handles this
    # configuration. So it's defined here if the defaults change.
    stream_options = dict( timeout=None, block=True, heartbeat_timeout=90 )

    # Reconnect timer list [1sec, 5sec, 10sec, 1min, 2min]
    re_timer = [1,2,10,60,120]
    re_timer_idx = 0

    while True:
        try:
            if name == "twitter":
                #home timeline stream
                stream = TwitterStream(auth=OAuth(
                        oauth_token, oauth_secret, CONSUMER_KEY, CONSUMER_SECRET),
                        domain="userstream.twitter.com", **stream_options)
                if home_replies:
                    tweet_iter = stream.user(replies="all")
                else:
                    tweet_iter = stream.user()
            else:
                h = html.parser.HTMLParser()
                stream = TwitterStream(auth=OAuth(
                        oauth_token, oauth_secret, CONSUMER_KEY, CONSUMER_SECRET),
                    **stream_options)

                twitter = Twitter(auth=OAuth(
                    oauth_token, oauth_secret, CONSUMER_KEY, CONSUMER_SECRET))

                if stream_args[0] != "&":
                    follow = stream_args[0]
                    twitter_data = twitter.users.lookup(screen_name=follow)
                    follow_ids = ""
                    for user in twitter_data:
                        follow_ids += user['id_str'] + ","
                    follow_ids = follow_ids[:-1]
                    if len(stream_args) == 2:
                        track = stream_args[1]
                        tweet_iter = stream.statuses.filter(track=track,follow=follow_ids)
                    else:
                        tweet_iter = stream.statuses.filter(follow=follow_ids)
                else:
                    track = stream_args[1]
                    tweet_iter = stream.statuses.filter(track=track)
        except:
            stream_end_message = "Connection problem (could not connect to twitter)"
            break;

        stream_end_message = "Unknown reason"

        # Iterate over the stream.
        for tweet in tweet_iter:
            # You must test that your tweet has text. It might be a delete
            # or data message.
            if tweet is None:
                stream_end_message = "'None' reply"
            elif tweet is Timeout:
                stream_end_message = "Timeout"
            elif tweet is HeartbeatTimeout:
                stream_end_message = "Heartbeat Timeout"
            elif tweet is Hangup:
                stream_end_message = "Hangup"
            elif tweet.get('text'):
                tweet = trim_tweet_data([tweet],screen_name,alt_rt_style,is_py3)
                client = connect()
                client.sendall(bytes(str(tweet),"utf-8"))
                client.close()
                stream_end_message = "Text message"
                # Reset the reconnect timer index when we get a new message
                re_timer_idx = 0
            else:
                #Got a other type of message
                client = connect()
                client.sendall(bytes(str(tweet),"utf-8"))
                client.close()
                stream_end_message = "Unhandled type message"

        client = connect()
        client.sendall(bytes('"Disconnected, trying to reconnect in {0} sec. Reason: {1}"'.format(re_timer[re_timer_idx], stream_end_message),"utf-8"))
        client.close()

        time.sleep(re_timer[re_timer_idx])
        if re_timer_idx < (len(re_timer) - 1):
            re_timer_idx += 1

    return "Stream shut down after: " + stream_end_message + ". You'll have to restart the stream manually. (:re_home, if home stream)"

def stream_close_cb(name,buffer):
    global sock_fd_dict
    global proc_hooks
    weechat.unhook(sock_hooks[name])
    #Is the process already unhooked?
    if proc_hooks[name]:
        weechat.unhook(proc_hooks[name])

    #remove fd key
    for key, value in sock_fd_dict.items():
        if value == name:
            sock_fd_dict.pop(key, None)
            break
    sock_fd_dict[name].close()
    proc_hooks.pop(name,None)
    sock_fd_dict.pop(name, None)
    return weechat.WEECHAT_RC_OK

def create_stream(name, args = ""):
    global sock_fd_dict
    global proc_hooks
    global sock_hooks

    if proc_hooks.get(name):
        return "Stream has already been created, close it before trying to open a new one"

    #Check if buffer exists
    buffer = weechat.buffer_search("python", name)
    if buffer == "":
        buffer = weechat.buffer_new(name, "buffer_input_cb", name, "stream_close_cb", name)
        setup_buffer(buffer)

    if not sock_fd_dict.get(name):
        file_name = tempfile.gettempdir() + "/we_tw_" + getpass.getuser() + "_" + name
        if os.path.exists(file_name):
            os.remove(file_name)

        server = socket.socket( socket.AF_UNIX, socket.SOCK_STREAM )
        server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        server.bind(file_name)
        #Don't block, timeout if no data is present
        server.setblocking(False)
        server.listen(1)
        file_fd = server.fileno()
        sock_fd_dict[str(file_fd)] = name
        sock_fd_dict[name] = server
        sock_hooks[name] = weechat.hook_fd(file_fd, 1, 0, 0, "twitter_stream_cb", buffer)

    proc_hooks[name] = weechat.hook_process("python3 " + SCRIPT_FILE_PATH + " " +
                "stream " + file_name + " " + args,  0 , "my_process_cb", str([buffer,"Stream"]))
    return "Started stream"

def my_process_cb(data, command, rc, out, err):

    if rc == weechat.WEECHAT_HOOK_PROCESS_ERROR:
        weechat.prnt("", "Error with command '%s'" %
                command.replace(script_options["oauth_token"],"").replace(script_options["oauth_secret"],""))
        return weechat.WEECHAT_RC_OK

    data = ast.literal_eval(data)
    buffer = data[0]
    end_mes = data[1]

    if out != "":
        if out[0] != "[" and out[0] != "{":
            #If message is just a string print it
            weechat.prnt(buffer, "%s%s" % (weechat.prefix("network"), out))
            process_output = []
        else:
            process_output = ast.literal_eval(out)

        #List message
        if len(end_mes) >= 1 and end_mes[0] == "L":
            if isinstance(process_output[-1], int):
                t_id = dict_tweet(str(process_output[-1])) + "\t"
                process_output = process_output[:-1]
                more = " ..."
            else:
                t_id = weechat.prefix("network")
                more = ""

            for nick in process_output:
                if end_mes == "LYFollowing":
                    end_mes = "LYou are following"
                    add_to_nicklist(buffer,nick)
                elif script_options['tweet_nicks']:
                    add_to_nicklist(buffer,nick,tweet_nicks_group[buffer])
            weechat.prnt_date_tags(buffer, 0, "no_highlight",
                    "%s%s: %s%s" % (t_id, end_mes[1:], process_output, more))
            return weechat.WEECHAT_RC_OK

        if end_mes == "About":
            weechat.prnt(buffer, "Nick: %s | Name: %s | Protected: %s" % (process_output['screen_name'],
                                                                        process_output['name'],
                                                                        process_output['protected']))
            weechat.prnt(buffer, "Description: %s" % process_output['description'])
            weechat.prnt(buffer, "Location: %s | Time zone: %s" % (process_output['location'], process_output['time_zone']))
            weechat.prnt(buffer, "Created at: %s | Verified user: %s" % (process_output['created_at'], process_output['verified']))
            weechat.prnt(buffer, "Following: %s | Followers: %s | Favourites: %s | Tweets: %s" % (process_output['friends_count'],
                                                                                               process_output['followers_count'],
                                                                                               process_output['favourites_count'],
                                                                                               process_output['statuses_count']))
            weechat.prnt(buffer, "Are you currently following this person: %s" % (process_output['following']))
            return weechat.WEECHAT_RC_OK

        elif end_mes == "Stream":
            global proc_hooks
            #Clean up the stream hooks
            name = weechat.buffer_get_string(buffer, "name")
            #Set the hook to none because it has already been unhooked
            proc_hooks[name] = None
            stream_close_cb(name, buffer)
            #TODO restart stream correctly
            #create_stream(name)
            return weechat.WEECHAT_RC_OK

        print_tweet_data(buffer,process_output,end_mes)

        if end_mes != "id" and end_mes != "":
            weechat.prnt(buffer, "%s%s" % (weechat.prefix("network"), end_mes))
    if err != "":
        weechat.prnt("", "stderr: %s" % err)
    return weechat.WEECHAT_RC_OK

def get_twitter_data(cmd_args):
    # Read the oauth token and auth with the twitter api.
    # Return the requested tweets
    no_home_replies = True
    alt_rt_style = False
    is_py3 = False
    screen_name = ""

    h = html.parser.HTMLParser()

    try:
        if cmd_args[-1][0] == "[":
            option_list = ast.literal_eval(cmd_args[-1])
            cmd_args.pop(-1)
            if "home_replies" in option_list:
                no_home_replies = False
            if "alt_rt_style" in option_list:
                alt_rt_style = True
            if "is_py3" in option_list:
                is_py3 = True
            screen_name = option_list[0]
    except:
        pass

    if len(cmd_args) < 4:
        return "Invalid command"

    oauth_token = cmd_args[1]
    oauth_secret= cmd_args[2]

    try:
        if cmd_args[3] == "auth":
            twitter = Twitter(
                auth=OAuth(oauth_token, oauth_secret, CONSUMER_KEY, CONSUMER_SECRET),
                format='', api_version=None)
            if len(cmd_args) == 5:
                #pin
                return twitter.oauth.access_token(oauth_verifier=cmd_args[4])
            else:
                return twitter.oauth.request_token()

        twitter = Twitter(auth=OAuth(
            oauth_token, oauth_secret, CONSUMER_KEY, CONSUMER_SECRET))

        if cmd_args[3] == "settings":
            #this only gets called from within weechat
                return twitter.account.settings()['screen_name']
        elif cmd_args[3] == "u":
            kwargs = dict(count=20, screen_name=cmd_args[4])
            if len(cmd_args) == 7:
                kwargs['count'] = int(cmd_args[6])
                kwargs['max_id'] = cmd_args[5]
            elif len(cmd_args) == 6:
                if int(cmd_args[5]) <= 200:
                    kwargs['count'] = int(cmd_args[5])
                else:
                    kwargs['max_id'] = cmd_args[5]
            tweet_data = twitter.statuses.user_timeline(**kwargs)
        elif cmd_args[3] == "r":
            if len(cmd_args) == 6:
                kwargs = dict(count=int(cmd_args[5]), max_id=cmd_args[4])
                tweet_data = twitter.statuses.mentions_timeline(**kwargs)
            elif len(cmd_args) == 5:
                if int(cmd_args[4]) <= 200:
                    tweet_data = twitter.statuses.mentions_timeline(count=int(cmd_args[4]))
                else:
                    tweet_data = twitter.statuses.mentions_timeline(max_id=cmd_args[4])
            else:
                tweet_data = twitter.statuses.mentions_timeline()
        elif cmd_args[3] == "v":
            tweet_data = [twitter.statuses.show._(cmd_args[4])()]
        elif cmd_args[3] == "th":
            #only fetch up to 20 tweets in one go
            tweets_left = 0
            tweet_id = cmd_args[4]
            tweet_data = []
            while tweets_left < 20:
                tweets_left += 1
                #TODO handle deleted tweets
                temp_tweet = twitter.statuses.show._(tweet_id)()
                tweet_data.append(temp_tweet)
                if temp_tweet["in_reply_to_status_id_str"] != None:
                    tweet_id = temp_tweet["in_reply_to_status_id_str"]
                else:
                    break;
        elif cmd_args[3] == "l":
            tweet_data = [twitter.statuses.show._(cmd_args[4])()]
            output = "Link for tweet: https://twitter.com/{}/status/{}".format(tweet_data[0]['user']['screen_name'],
                                                                               tweet_data[0]['id_str'])
            return output
        elif cmd_args[3] == "rt":
            tweet_data = [twitter.statuses.retweet._(cmd_args[4])()]
            #The home stream prints you messages as well...
            # TODO add a switch to print this if the user has deatived the home timeline stream.
            # For all commands like this!
            tweet_data = []
        elif cmd_args[3] == "d":
            #deletes tweet made by the user _(...) converts the id string to a call
            #returns the tweet that was deleted (not a list(dict) just a dict)
            #make it into a list so we don't have to write special cases for this
            tweet_data = [twitter.statuses.destroy._(cmd_args[4])()]
        elif cmd_args[3] == "t":
            #returns the tweet that was sent (not a list(dict) just a dict)
            #make it into a list so we don't have to write special cases for this
            tweet_data = [twitter.statuses.update(status=h.unescape(cmd_args[4]))]
            #The home stream prints you messages as well...
            tweet_data = []
        elif cmd_args[3] == "re":
            tweet_data = [twitter.statuses.update(status=h.unescape(cmd_args[5]),
                in_reply_to_status_id=cmd_args[4])]
            #The home stream prints you messages as well...
            tweet_data = []
        elif cmd_args[3] == "new":
            tweet_data = twitter.statuses.home_timeline(since_id = cmd_args[4], count=200, exclude_replies = no_home_replies)
            if tweet_data == []:
                return "No new tweets available."
        elif cmd_args[3] == "follow":
            tweet_data = []
            twitter.friendships.create(screen_name = cmd_args[4])
        elif cmd_args[3] == "unfollow":
            tweet_data = []
            twitter.friendships.destroy(screen_name = cmd_args[4])
        elif cmd_args[3] == "f" or cmd_args[3] == "fo":
            if len(cmd_args) == 6:
                kwargs = dict(screen_name = cmd_args[4], stringify_ids = True, cursor = int(cmd_args[5]), count = 250)
            else:
                kwargs = dict(screen_name = cmd_args[4], stringify_ids = True, cursor = -1, count = 250)
            if cmd_args[3] == "f":
                tweet_data = twitter.friends.ids(**kwargs)
            else:
                tweet_data = twitter.followers.ids(**kwargs)
            kwargs['cursor'] = tweet_data['next_cursor']
            friend_ids = tweet_data['ids']
            friend_list = list()

            while len(friend_ids) > 100:
                tweet_data = twitter.users.lookup(user_id=",".join(friend_ids[:100]))
                friend_ids = friend_ids[100:]
                for user in tweet_data:
                    friend_list.append(user['screen_name'])
            tweet_data = twitter.users.lookup(user_id=",".join(friend_ids))
            for user in tweet_data:
                friend_list.append(user['screen_name'])

            if kwargs['cursor'] != 0:
                friend_list.append(kwargs['cursor'])
            return friend_list
        elif cmd_args[3] == "a":
            return twitter.users.show(screen_name = cmd_args[4])
        elif cmd_args[3] == "b":
            tweet_data = []
            twitter.blocks.create(screen_name = cmd_args[4])
        elif cmd_args[3] == "ub":
            tweet_data = []
            twitter.blocks.destroy(screen_name = cmd_args[4])
        elif cmd_args[3] == "blocks":
            tweet_data = twitter.blocks.list(skip_status = True)
            block_list = list()
            for user in tweet_data['users']:
                block_list.append(user['screen_name'])
            return block_list
        elif cmd_args[3] == "fav":
            tweet_data = [twitter.favorites.create(_id=cmd_args[4])]
        elif cmd_args[3] == "unfav":
            tweet_data = [twitter.favorites.destroy(_id=cmd_args[4])]
        elif cmd_args[3] == "favs":
            if len(cmd_args) >= 5:
                kwargs = dict()
                if not cmd_args[4].isdigit():
                    kwargs['screen_name'] = cmd_args[4]
                    cmd_args.pop(4)
                if len(cmd_args) == 5:
                    if int(cmd_args[4]) <= 200:
                        kwargs['count'] = int(cmd_args[4])
                    else:
                        kwargs['max_id'] = cmd_args[4]
                elif len(cmd_args) == 6:
                    kwargs['count'] = int(cmd_args[5])
                    kwargs['max_id'] = cmd_args[4]
                tweet_data = twitter.favorites.list(**kwargs)
            else:
                tweet_data = twitter.favorites.list()
        elif cmd_args[3] == "fart":
            twitter.favorites.create(_id=cmd_args[4])
            tweet_data = [twitter.statuses.retweet._(cmd_args[4])()]
            tweet_data = []
        elif cmd_args[3] == "limits":
            output = ""
            if len(cmd_args) >= 5:
                tweet_data = twitter.application.rate_limit_status(resources=",".join(cmd_args[4:]))
            else:
                tweet_data = twitter.application.rate_limit_status()
            for res in tweet_data['resources']:
                output += res + ":\n"
                for sub_res in tweet_data['resources'][res]:
                    output += "  " + sub_res[len(res)+2:] + ":\n"
                    output += "    " + 'reset' + ": " + time.strftime('%Y-%m-%d %H:%M:%S',
                            time.localtime(tweet_data['resources'][res][sub_res]['reset'])) + "\n"
                    output += "    " + 'limit' + ": " + str(tweet_data['resources'][res][sub_res]['limit']) + "\n"
                    output += "    " + 'remaining' + ": " + str(tweet_data['resources'][res][sub_res]['remaining']) + "\n"
            return output
        elif cmd_args[3] == "home":
            if len(cmd_args) == 6:
                kwargs = dict(count=int(cmd_args[5]), max_id=cmd_args[4], exclude_replies = no_home_replies)
                tweet_data = twitter.statuses.home_timeline(**kwargs)
            elif len(cmd_args) == 5:
                if int(cmd_args[4]) <= 200:
                    tweet_data = twitter.statuses.home_timeline(count=int(cmd_args[4]), exclude_replies = no_home_replies)
                else:
                    tweet_data = twitter.statuses.home_timeline(max_id=cmd_args[4], exclude_replies = no_home_replies)
            else:
                tweet_data = twitter.statuses.home_timeline(exclude_replies = no_home_replies)
        else:
            return "Invalid command: " + cmd_args[3]
    except TwitterHTTPError as err:
        #See if we can print a pretty error message first
        data = err.response_data
        if "errors" in data and "message" in data["errors"][0]:
            return "Error: " + data["errors"][0]["message"]
        else:
            return "Unexpected error in get_twitter_data:%s\n Call: %s" % (sys.exc_info(), cmd_args[3])
    except:
        return "Unexpected error in get_twitter_data:%s\n Call: %s" % (sys.exc_info(), cmd_args[3])

    return trim_tweet_data(tweet_data,screen_name,alt_rt_style,is_py3)

# callback for data received in input
def buffer_input_cb(data, buffer, input_data):
    # ...
    end_message = ""
    options = [script_options['screen_name']]
    if script_options['alt_rt_style']:
        options.append("alt_rt_style")
    if script_options['home_replies']:
        options.append("home_replies")
    if is_py3:
        options.append("is_py3")

    if input_data[0] == ':':
        if data != "silent":
            weechat.prnt_date_tags(buffer, 0, "no_highlight", input_data)
        input_args = input_data.split()
        command = input_args[0][1:]
        if command_dict.get(command):
            input_data = input_data.replace(command,command_dict[command],1)
            command = command_dict[command]
        if command == 'd' and tweet_dict.get(input_args[1]):
            input_data = 'd ' + tweet_dict[input_args[1]]
            weechat.prnt(buffer, "%sYou deleted the following tweet:" % weechat.prefix("network"))
        elif command == 'v' and tweet_dict.get(input_args[1]):
            input_data = 'v ' + tweet_dict[input_args[1]]
            end_message = "Done"
        elif command == 'th' and tweet_dict.get(input_args[1]):
            input_data = 'th ' + tweet_dict[input_args[1]]
            end_message = "Done"
        elif command == 'l' and tweet_dict.get(input_args[1]):
            input_data = 'l ' + tweet_dict[input_args[1]]
        elif command == 'rt' and tweet_dict.get(input_args[1]):
            end_message = "id"
            input_data = 'rt ' + tweet_dict[input_args[1]]
        elif command == 're' and tweet_dict.get(input_args[1]):
            end_message = "id"
            input_data = 're ' + tweet_dict[input_args[1]] + " '" + html_escape(input_data[6:]) + "'"
        elif command == 'new':
            end_message = "id"
            if script_options['last_id'] != "":
                input_data = 'new ' + script_options['last_id']
            else:
                input_data = 'home'
        elif command == 'home' or command == 'r' or (command == 'favs' and len(input_args) >= 2 and input_args[1].isdigit()):
            input_data = command
            if len(input_args) == 3 and tweet_dict.get(input_args[1]) and input_args[2].isdigit():
                num = int(input_args[2])
                # 200 tweets is the max request limit
                if num <= 200 and num > 0:
                    input_data += " " + tweet_dict[input_args[1]] + " " + input_args[2]
                else:
                    input_data += " " + tweet_dict[input_args[1]]
            elif len(input_args) == 2:
                if tweet_dict.get(input_args[1]):
                    input_data += " " + tweet_dict[input_args[1]]
                elif input_args[1].isdigit():
                    num = int(input_args[1])
                    # 200 tweets is the max request limit
                    if num <= 200 and num > 0:
                        input_data += " " + input_args[1]
            end_message = "Done"
        elif command == 'u' or (command == 'favs' and len(input_args) >= 3):
            input_data = " ".join(input_args[:2])[1:]
            if len(input_args) == 4 and tweet_dict.get(input_args[2]) and input_args[3].isdigit():
                num = int(input_args[3])
                # 200 tweets is the max request limit
                if num <= 200 and num > 0:
                    input_data += " " + tweet_dict[input_args[2]] + " " + input_args[3]
                else:
                    input_data += " " + tweet_dict[input_args[2]]
            elif len(input_args) == 3:
                if tweet_dict.get(input_args[2]):
                    input_data += " " + tweet_dict[input_args[2]]
                elif input_args[2].isdigit():
                    num = int(input_args[2])
                    # 200 tweets is the max request limit
                    if num <= 200 and num > 0:
                        input_data += " " + input_args[2]
            end_message = "Done"
        elif command == 'auth':
            if len(input_args) == 2:
                oauth_dance(buffer,input_args[1])
            else:
                oauth_dance(buffer)
            return weechat.WEECHAT_RC_OK
        elif command == 'f' or command == 'fo':
            #L because we are returning a list to be printed later on
            end_message = "L"
            if len(input_args) == 3 and tweet_dict.get(input_args[2]):
                input_data = command + " " + input_args[1] + " " + tweet_dict[input_args[2]]
            elif len(input_args) == 2:
                if tweet_dict.get(input_args[1]):
                    input_data = command + " " + script_options['screen_name'] + " " + tweet_dict[input_args[1]]
                    #Your list, not any other users
                    end_message += "Y"
                else:
                    input_data = input_data[1:]
            else:
                input_data = command + " " + script_options['screen_name']
                end_message += "Y"
            if command == 'f':
                end_message += "Following"
            else:
                end_message += "Followers"
        elif command == 'a':
            input_data = input_data[1:]
            end_message = "About"
        elif command == 'blocks':
            input_data = input_data[1:]
            end_message = "LBlock list"
        elif command == 'fav' and tweet_dict.get(input_args[1]):
            input_data = 'fav ' + tweet_dict[input_args[1]]
            weechat.prnt(buffer, "%sYou fave'd the following tweet:" % weechat.prefix("network"))
        elif command == 'unfav' and tweet_dict.get(input_args[1]):
            input_data = 'unfav ' + tweet_dict[input_args[1]]
            weechat.prnt(buffer, "%sYou unfave'd the following tweet:" % weechat.prefix("network"))
        elif command == 'fart' and tweet_dict.get(input_args[1]):
            end_message = "id"
            input_data = 'fart ' + tweet_dict[input_args[1]]
            weechat.prnt(buffer, "%sYou fave'd and retweeted the following tweet:" % weechat.prefix("network"))
        elif command == 'cnicks':
            global tweet_nicks_group
            if tweet_nicks_group[buffer] != "":
                weechat.nicklist_remove_group(buffer, tweet_nicks_group[buffer])
                tweet_nicks_group[buffer] = ""
            tweet_nicks_group[buffer] = weechat.nicklist_add_group(buffer, "", "Tweet_parse",
                    "weechat.color.nicklist_group", 1)
            return weechat.WEECHAT_RC_OK
        elif command == 'help':
            weechat.command(buffer,"/help twitter")
            weechat.prnt(buffer, "Exec command /help twitter, check your root buffer")
            return weechat.WEECHAT_RC_OK
        elif command == 'clear':
            weechat.command(buffer,"/buffer clear")
            return weechat.WEECHAT_RC_OK
        elif command == 'stream':
            args = html_escape(input_data[7:])
            weechat.prnt(buffer,create_stream("t_stream",args))
            return weechat.WEECHAT_RC_OK
        elif command == 're_home':
            weechat.prnt(buffer,create_stream("twitter"))
            return weechat.WEECHAT_RC_OK
        else:
            input_data = input_data[1:]
            end_message = "Done"
    else:
        end_message = "id"
        #esacpe special chars when printing to commandline
        input_data = 't ' + "'" + html_escape(input_data) + "'"
        #input_data = 't ' + "'" + html.escape(input_data) + "'"

    weechat.hook_process("python3 " + SCRIPT_FILE_PATH + " " +
                script_options["oauth_token"] + " " + script_options["oauth_secret"] + " " +
                input_data + " " + '"' + str(options) + '"', 10 * 1000, "my_process_cb", str([buffer,end_message]))
    return weechat.WEECHAT_RC_OK

def my_command_cb(data, buffer, args):
    # ...

    buffer_input_cb(data, twit_buf, ":"+args)

    return weechat.WEECHAT_RC_OK

def hook_commands_and_completions():
    compl_list = []
    com_list = []
    desc_list = []
    for command in sorted(command_dict):
        compl_list.append(command)
        com_list.append(command + weechat.color("*red") + " or " +
                weechat.color('reset') + command_dict[command] + "\n")
        desc_list.append(weechat.color("chat_nick_other") + command + ":    \n" + desc_dict[command])
    weechat.hook_command("twitter", "Command to interact with the twitter api/plugin",
        " | ".join(com_list),
        "You can type all of these command in the twitter buffer if you add a ':' before the command, IE:\n"
        ":limits\n\n"
        "If you don't type a command in the twitter buffer you will tweet that instead,\n"
        "text after 140 chars will turn red to let you know were twitter will cut off your tweet.\n\n"
        + weechat.color("*red") + "NOTE:\n"
        "There are limits on how many twitter api calls you can do, some calls are _quite_ restricted.\n"
        "So if you get HTML errors from the twitter lib you probably exceeded the limit\n"
        "you can check out your limits with the rate_limits/limits command.\n"
        "_Most_ commands in this plugin only uses one call. If you want to check old tweets\n"
        "in your home timeline it's better to request many tweets in one go.\n"
        "That way you don't have to request new tweets as often to go further back in the timeline.\n"
        "And thus you are less likely to hit the limit of requests you can do in the 15 min time window.\n"
        "\nYou can write newlines in your tweet with html newline '&#13;&#10;' (you can autocomplete it)\n"
        "\nThe 'number' next to the nicks in the chat window is the <id> of the tweet it's used\n"
        "in the some of the twitter plugin commands.\n\n"
        "Command desc:\n"+ "\n".join(desc_list),
        " || ".join(compl_list),
        "my_command_cb", "")

# callback called when buffer is closed
# TODO rewrite this so it unloads the plugin
def buffer_close_cb(data, buffer):
    # ...
    #TODO handle multiple buffers and free up global buffer pointers
    weechat.unhook_all()
    return weechat.WEECHAT_RC_OK

def tweet_length(message):
    """Replace URLs with placeholders, 20 for http URLs, 21 for https."""
    # regexes to match URLs
    octet = r'(?:2(?:[0-4]\d|5[0-5])|1\d\d|\d{1,2})'
    ip_addr = r'%s(?:\.%s){3}' % (octet, octet)
    # Base domain regex off RFC 1034 and 1738
    label = r'[0-9a-z][-0-9a-z]*[0-9a-z]?'
    domain = r'%s(?:\.%s)*\.[a-z][-0-9a-z]*[a-z]?' % (label, label)
    url_re = re.compile(r'(\w+://(?:%s|%s)(?::\d+)?(?:/[^\])>\s]*)?)' % \
            (domain, ip_addr), re.I)

    new_message = message

    for url in url_re.findall(message):
        short_url = 'x' * 20
        if url.startswith('https'):
            short_url = 'x' * 21
        new_message = new_message.replace(url, short_url)

    return len(new_message)

def my_modifier_cb(data, modifier, modifier_data, string):
    if not weechat.current_buffer() in friends_nicks_group:
        return string

    #check if this is a commandline argument
    if string == "" or string[0] == "/":
        return string

    length = tweet_length(string)

    # Subtract local command argument from length
    if string[:3] == ":re":
        #:re a2
        length = length - 6

    if length > 140:
        index = 140 - length
        string = string[:index] + weechat.color("*red") + string[index:]

    return string

def parse_oauth_tokens(result):
    for r in result.split('&'):
        k, v = r.split('=')
        if k == 'oauth_token':
            oauth_token = v
        elif k == 'oauth_token_secret':
            oauth_token_secret = v
    return oauth_token, oauth_token_secret

def oauth_proc_cb(data, command, rc, out, err):

    global script_options
    buffer = twit_buf

    if rc == weechat.WEECHAT_HOOK_PROCESS_ERROR:
        weechat.prnt("", "Error with command '%s'" %
                command.replace(script_options["oauth_token"],"").replace(script_options["oauth_secret"],""))
        return weechat.WEECHAT_RC_OK

    if len(out) > 16 and out[:16] == "Unexpected error":
        #Auth command failed (most likely connection problems)
        weechat.prnt("", out);
        weechat.prnt(buffer, "An error occured, check root buffer. Most likely a connection problem, fix it and the reload weetweet");
        return weechat.WEECHAT_RC_OK

    if out != "":
        if data == "nick":
            weechat.config_set_plugin('screen_name', out.strip())
            finish_init()
        elif data == "friends":
            process_output = ast.literal_eval(out)
            if isinstance(process_output[-1], int):
                t_id = dict_tweet(str(process_output[-1])) + "\t"
                process_output = process_output[:-1]
                weechat.prnt_date_tags(buffer, 0, "no_highlight", t_id +
                    "It seems like you are following more than 250 people. Due to twitter api limits " +
                    "it is nearly impossible to get large groups of followers in one go. However the " +
                    "nicks will be added when they tweet something so if you don't have to be able " +
                    "autocomplete them from the start this is not a problem for you." +
                    " If you want to get the rest of the nicks you can use the id of this text.")

            for nick in process_output:
                add_to_nicklist(buffer,nick)
        elif data == "auth1":
            #First auth step to request pin code
            oauth_token, oauth_token_secret = parse_oauth_tokens(out)
            script_options['oauth_token'] = oauth_token
            script_options['oauth_secret'] = oauth_token_secret
            weechat.prnt(buffer,"""
    Copy the PIN number that appears on the linked web page and type ":auth <pin>"
    in weechat. For example ":auth 123456"
    """)
            oauth_url = ('https://api.twitter.com/oauth/authorize?oauth_token=' +
                     oauth_token)
            weechat.prnt(buffer," Please go here to get your PIN: " + oauth_url)
        elif data == "auth2":
            oauth_token, oauth_token_secret = parse_oauth_tokens(out)
            weechat.config_set_plugin('oauth_token', oauth_token)
            weechat.config_set_plugin('oauth_secret', oauth_token_secret)
            weechat.config_set_plugin('auth_complete', "on")
            weechat.prnt(buffer," Done! now you can begin using this script!")
            weechat.hook_process("python3 " + SCRIPT_FILE_PATH + " " +
                    script_options["oauth_token"] + " " + script_options["oauth_secret"] + " " +
                    "settings []", 10 * 1000, "oauth_proc_cb", "nick")
    return weechat.WEECHAT_RC_OK

def oauth_dance(buffer, pin = ""):
    #Auth the twitter client
    if pin == "":
        weechat.prnt(buffer,"Hi there! We're gonna get you all set up to use this plugin.")
        weechat.hook_process("python3 " + SCRIPT_FILE_PATH + " " + "'' " + "'' " +
                "auth", 10 * 1000, "oauth_proc_cb", "auth1")
    else:
        oauth_verifier = pin.strip()
        weechat.hook_process("python3 " + SCRIPT_FILE_PATH + " " +
                script_options["oauth_token"] + " " + script_options["oauth_secret"] + " " +
                "auth "+ oauth_verifier, 10 * 1000, "oauth_proc_cb", "auth2")

def setup_buffer(buffer):
    # set title
    weechat.buffer_set(buffer, "title", "Twitter buffer, type ':help' for options.")

    # disable logging, by setting local variable "no_log" to "1"
    weechat.buffer_set(buffer, "localvar_set_no_log", "1")

    #create main nicklist
    friends_nicks_group[buffer] = weechat.nicklist_add_group(buffer, "", "Friends",
                "weechat.color.nicklist_group", 1)

    tweet_nicks_group[buffer] = weechat.nicklist_add_group(buffer, "", "Tweet_parse",
                "weechat.color.nicklist_group", 1)
    autocomp_group = weechat.nicklist_add_group(buffer, "", "Autocomp",
                "weechat.color.nicklist_group", 1)
    #newline autocomplete
    weechat.nicklist_add_nick(buffer, autocomp_group, "&#13;&#10;", 'bar_fg', '', '', 1)

    #show nicklist
    weechat.buffer_set(buffer, "nicklist", "1")

    user_nick = script_options['screen_name']

    weechat.buffer_set(buffer, "localvar_set_nick", user_nick)

    add_to_nicklist(buffer, user_nick)
    # Highlight user nick
    weechat.buffer_set(buffer, "highlight_words", user_nick)

def finish_init():
    buffer = twit_buf

    if script_options['screen_name'] == "":
         weechat.hook_process("python3 " + SCRIPT_FILE_PATH + " " +
                script_options["oauth_token"] + " " + script_options["oauth_secret"] + " " +
                "settings []", 10 * 1000, "oauth_proc_cb", "nick")
         return
    setup_buffer(buffer)

    #Add friends to nick list
    weechat.hook_process("python3 " + SCRIPT_FILE_PATH + " " +
           script_options["oauth_token"] + " " + script_options["oauth_secret"] + " " +
           "f " + script_options['screen_name'] + " []", 10 * 1000, "oauth_proc_cb", "friends")

if __name__ == "__main__" and weechat_call:
    weechat.register( SCRIPT_NAME , "DarkDefender", "1.2.7", "GPL3", "Weechat twitter client", "", "")

    if not import_ok:
        weechat.prnt("", "Can't load twitter python lib >= " + required_twitter_version)
        weechat.prnt("", "Install it via your package manager or go to http://mike.verdone.ca/twitter/")
    else:
        hook_commands_and_completions()

        #Set register script options if not available

        for option, default_value in script_options.items():
            if not weechat.config_is_set_plugin(option):
                if isinstance(default_value,bool):
                    if default_value:
                        default_value = "on"
                    else:
                        default_value = "off"
                weechat.config_set_plugin(option, default_value)

        read_config()
        #Hook for config changes
        weechat.hook_config("plugins.var.python." + SCRIPT_NAME + ".*", "config_cb", "")

        #Create buffer
        twit_buf = weechat.buffer_new("twitter", "buffer_input_cb", "", "buffer_close_cb", "")

        #Hook text input so we can update the bar item
        weechat.hook_modifier("input_text_display", "my_modifier_cb", "")

        if script_options['auth_complete']:
            finish_init()
            #Create home_timeline stream.
            #This will indirectly trigger the ":new" command if stream sucessfully starts
            create_stream("twitter")
        else:
            weechat.prnt(twit_buf,"""You have to register this plugin with twitter for it to work.
Type ":auth" and follow the instructions to do that""")

elif import_ok:
    if sys.argv[1] == "stream":
        print(twitter_stream(sys.argv))
    else:
        print(get_twitter_data(sys.argv))
else:
    print("Can't load twitter python lib >= " + required_twitter_version )




# -*- coding: utf-8 -*-
# mnotify.py
# Copyleft (c) 2013 maker <maker@python.it>
# Released under Beerware License
#
# based on:
# anotify.py
# Copyright (c) 2012 magnific0 <jacco.geul@gmail.com>
#
# based on:
# growl.py
# Copyright (c) 2011 Sorin Ionescu <sorin.ionescu@gmail.com>


# -----------------------------------------------------------------------------
# Imports
# -----------------------------------------------------------------------------
import re
import os
import subprocess
from email.mime.text import MIMEText
#import smtplib

import weechat

SCRIPT_NAME = 'mnotify'
SCRIPT_AUTHOR = 'maker'
SCRIPT_VERSION = '0.3'
SCRIPT_LICENSE = 'Beerware License'
SCRIPT_DESC = 'Sends mail notifications upon events.'

# -----------------------------------------------------------------------------
# Settings
# -----------------------------------------------------------------------------
SETTINGS = {
    'show_public_message': 'off',
    'show_private_message': 'on',
    'show_public_action_message': 'off',
    'show_private_action_message': 'on',
    'show_notice_message': 'off',
    'show_invite_message': 'on',
    'show_highlighted_message': 'on',
    'show_server': 'on',
    'show_channel_topic': 'on',
    'show_dcc': 'on',
    'show_upgrade_ended': 'on',
    'sticky': 'off',
    'sticky_away': 'on',
    'sendmail': 'msmtp',
    'email_to': '',
    'email_from': 'irc <irc@localhost>'
}



# -----------------------------------------------------------------------------
# Globals
# -----------------------------------------------------------------------------
TAGGED_MESSAGES = {
    'public message or action': set(['irc_privmsg', 'notify_message']),
    'private message or action': set(['irc_privmsg', 'notify_private']),
    'notice message': set(['irc_notice', 'notify_private']),
    'invite message': set(['irc_invite', 'notify_highlight']),
    'channel topic': set(['irc_topic', ]),
    #'away status': set(['away_info', ]),
}


UNTAGGED_MESSAGES = {
    'away status':
        re.compile(r'^You ((\w+).){2,3}marked as being away', re.UNICODE),
    'dcc chat request':
        re.compile(r'^xfer: incoming chat request from (\w+)', re.UNICODE),
    'dcc chat closed':
        re.compile(r'^xfer: chat closed with (\w+)', re.UNICODE),
    'dcc get request':
        re.compile(
            r'^xfer: incoming file from (\w+) [^:]+: ((?:,\w|[^,])+),',
            re.UNICODE),
    'dcc get completed':
        re.compile(r'^xfer: file ([^\s]+) received from \w+: OK', re.UNICODE),
    'dcc get failed':
        re.compile(
            r'^xfer: file ([^\s]+) received from \w+: FAILED',
            re.UNICODE),
    'dcc send completed':
        re.compile(r'^xfer: file ([^\s]+) sent to \w+: OK', re.UNICODE),
    'dcc send failed':
        re.compile(r'^xfer: file ([^\s]+) sent to \w+: FAILED', re.UNICODE),
}


DISPATCH_TABLE = {
    'away status': 'set_away_status',
    'public message or action': 'notify_public_message_or_action',
    'private message or action': 'notify_private_message_or_action',
    'notice message': 'notify_notice_message',
    'invite message': 'notify_invite_message',
    'channel topic': 'notify_channel_topic',
    'dcc chat request': 'notify_dcc_chat_request',
    'dcc chat closed': 'notify_dcc_chat_closed',
    'dcc get request': 'notify_dcc_get_request',
    'dcc get completed': 'notify_dcc_get_completed',
    'dcc get failed': 'notify_dcc_get_failed',
    'dcc send completed': 'notify_dcc_send_completed',
    'dcc send failed': 'notify_dcc_send_failed',
}


STATE = {
    'icon': None,
    'is_away': False
}


# -----------------------------------------------------------------------------
# Notifiers
# -----------------------------------------------------------------------------
def cb_irc_server_connected(data, signal, signal_data):
    '''Notify when connected to IRC server.'''
    if weechat.config_get_plugin('show_server') == 'on':
        a_notify(
            'Server',
            'Server Connected',
            'Connected to network {0}.'.format(signal_data))
    return weechat.WEECHAT_RC_OK


def cb_irc_server_disconnected(data, signal, signal_data):
    '''Notify when disconnected to IRC server.'''
    if weechat.config_get_plugin('show_server') == 'on':
        a_notify(
            'Server',
            'Server Disconnected',
            'Disconnected from network {0}.'.format(signal_data))
    return weechat.WEECHAT_RC_OK


def cb_notify_upgrade_ended(data, signal, signal_data):
    '''Notify on end of WeeChat upgrade.'''
    if weechat.config_get_plugin('show_upgrade_ended') == 'on':
        a_notify(
            'WeeChat',
            'WeeChat Upgraded',
            'WeeChat has been upgraded.')
    return weechat.WEECHAT_RC_OK


def notify_highlighted_message(buffername, prefix, message):
    '''Notify on highlighted message.'''
    if weechat.config_get_plugin("show_highlighted_message") == "on":
        a_notify(
            'Highlight',
            'Highlighted on {0} by {1}'.format(buffername, prefix),
            "{0}: {1}".format(prefix, message),
        )


def notify_public_message_or_action(buffername, prefix, message, highlighted):
    '''Notify on public message or action.'''
    if prefix == ' *':
        regex = re.compile(r'^(\w+) (.+)$', re.UNICODE)
        match = regex.match(message)
        if match:
            prefix = match.group(1)
            message = match.group(2)
            notify_public_action_message(buffername, prefix,
                                         message, highlighted)
    else:
        if highlighted:
            notify_highlighted_message(buffername, prefix, message)
        elif weechat.config_get_plugin("show_public_message") == "on":
            a_notify(
                'Public',
                'Public Message on {0}'.format(buffername),
                '{0}: {1}'.format(prefix, message))


def notify_private_message_or_action(buffername, prefix, message, highlighted):
    '''Notify on private message or action.'''
    regex = re.compile(r'^CTCP_MESSAGE.+?ACTION (.+)$', re.UNICODE)
    match = regex.match(message)
    if match:
        notify_private_action_message(buffername, prefix,
                                      match.group(1), highlighted)
    else:
        if prefix == ' *':
            regex = re.compile(r'^(\w+) (.+)$', re.UNICODE)
            match = regex.match(message)
            if match:
                prefix = match.group(1)
                message = match.group(2)
                notify_private_action_message(buffername, prefix,
                                              message, highlighted)
        else:
            if highlighted:
                notify_highlighted_message(buffername, prefix, message)
            elif weechat.config_get_plugin("show_private_message") == "on":
                a_notify(
                    'Private',
                    'Private Message',
                    '{0}: {1}'.format(prefix, message))


def notify_public_action_message(buffername, prefix, message, highlighted):
    '''Notify on public action message.'''
    if highlighted:
        notify_highlighted_message(buffername, prefix, message)
    elif weechat.config_get_plugin("show_public_action_message") == "on":
        a_notify(
            'Action',
            'Public Action Message on {0}'.format(buffername),
            '{0}: {1}'.format(prefix, message),
        )


def notify_private_action_message(buffername, prefix, message, highlighted):
    '''Notify on private action message.'''
    if highlighted:
        notify_highlighted_message(buffername, prefix, message)
    elif weechat.config_get_plugin("show_private_action_message") == "on":
        a_notify(
            'Action',
            'Private Action Message',
            '{0}: {1}'.format(prefix, message),
        )

def notify_notice_message(buffername, prefix, message, highlighted):
    '''Notify on notice message.'''
    regex = re.compile(r'^([^\s]*) [^:]*: (.+)$', re.UNICODE)
    match = regex.match(message)
    if match:
        prefix = match.group(1)
        message = match.group(2)
        if highlighted:
            notify_highlighted_message(buffername, prefix, message)
        elif weechat.config_get_plugin("show_notice_message") == "on":
            a_notify(
                'Notice',
                'Notice Message',
                '{0}: {1}'.format(prefix, message))


def notify_invite_message(buffername, prefix, message, highlighted):
    '''Notify on channel invitation message.'''
    if weechat.config_get_plugin("show_invite_message") == "on":
        regex = re.compile(
            r'^You have been invited to ([^\s]+) by ([^\s]+)$', re.UNICODE)
        match = regex.match(message)
        if match:
            channel = match.group(1)
            nick = match.group(2)
            a_notify(
                'Invite',
                'Channel Invitation',
                '{0} has invited you to join {1}.'.format(nick, channel))


def notify_channel_topic(buffername, prefix, message, highlighted):
    '''Notify on channel topic change.'''
    if weechat.config_get_plugin("show_channel_topic") == "on":
        regex = re.compile(
            r'^\w+ has (?:changed|unset) topic for ([^\s]+)' +
                '(?:(?: from "(?:.+)")? to "(.+)")?',
            re.UNICODE)
        match = regex.match(message)
        if match:
            channel = match.group(1)
            topic = match.group(2) or ''
            a_notify(
                'Channel',
                'Channel Topic on {0}'.format(buffername),
                "{0}: {1}".format(channel, topic))


def notify_dcc_chat_request(match):
    '''Notify on DCC chat request.'''
    if weechat.config_get_plugin("show_dcc") == "on":
        nick = match.group(1)
        a_notify(
            'DCC',
            'Direct Chat Request',
            '{0} wants to chat directly.'.format(nick))


def notify_dcc_chat_closed(match):
    '''Notify on DCC chat termination.'''
    if weechat.config_get_plugin("show_dcc") == "on":
        nick = match.group(1)
        a_notify(
            'DCC',
            'Direct Chat Ended',
            'Direct chat with {0} has ended.'.format(nick))


def notify_dcc_get_request(match):
    'Notify on DCC get request.'
    if weechat.config_get_plugin("show_dcc") == "on":
        nick = match.group(1)
        file_name = match.group(2)
        a_notify(
            'DCC',
            'File Transfer Request',
            '{0} wants to send you {1}.'.format(nick, file_name))


def notify_dcc_get_completed(match):
    'Notify on DCC get completion.'
    if weechat.config_get_plugin("show_dcc") == "on":
        file_name = match.group(1)
        a_notify('DCC', 'Download Complete', file_name)


def notify_dcc_get_failed(match):
    'Notify on DCC get failure.'
    if weechat.config_get_plugin("show_dcc") == "on":
        file_name = match.group(1)
        a_notify('DCC', 'Download Failed', file_name)


def notify_dcc_send_completed(match):
    'Notify on DCC send completion.'
    if weechat.config_get_plugin("show_dcc") == "on":
        file_name = match.group(1)
        a_notify('DCC', 'Upload Complete', file_name)


def notify_dcc_send_failed(match):
    'Notify on DCC send failure.'
    if weechat.config_get_plugin("show_dcc") == "on":
        file_name = match.group(1)
        a_notify('DCC', 'Upload Failed', file_name)


# -----------------------------------------------------------------------------
# Utility
# -----------------------------------------------------------------------------
def set_away_status(match):
    status = match.group(1)
    if status == 'been ':
        STATE['is_away'] = True
    if status == 'longer ':
        STATE['is_away'] = False


def cb_process_message(
    data,
    wbuffer,
    date,
    tags,
    displayed,
    highlight,
    prefix,
    message
):
    '''Delegates incoming messages to appropriate handlers.'''
    tags = set(tags.split(','))
    functions = globals()
    is_public_message = tags.issuperset(
        TAGGED_MESSAGES['public message or action'])
    buffer_name = weechat.buffer_get_string(wbuffer, 'name')
    dcc_buffer_regex = re.compile(r'^irc_dcc\.', re.UNICODE)
    dcc_buffer_match = dcc_buffer_regex.match(buffer_name)
    highlighted = False
    if int(highlight):
        highlighted = True
    # Private DCC message identifies itself as public.
    if is_public_message and dcc_buffer_match:
        notify_private_message_or_action(buffer_name, prefix,
                                         message, highlighted)
        return weechat.WEECHAT_RC_OK
    # Pass identified, untagged message to its designated function.
    for key, value in UNTAGGED_MESSAGES.items():
        match = value.match(message)
        if match:
            functions[DISPATCH_TABLE[key]](match)
            return weechat.WEECHAT_RC_OK
    # Pass identified, tagged message to its designated function.
    for key, value in TAGGED_MESSAGES.items():
        if tags.issuperset(value):
            functions[DISPATCH_TABLE[key]](buffer_name, prefix,
                                           message, highlighted)
            return weechat.WEECHAT_RC_OK
    return weechat.WEECHAT_RC_OK


def a_notify(notification, subject, message):
    if STATE['is_away'] and weechat.config_get_plugin('sticky_away') == 'off':
        return
    if not STATE['is_away'] and weechat.config_get_plugin('sticky') == 'off':
        return

    msg = MIMEText(message)
    msg['From'] = weechat.config_get_plugin('email_from')
    msg['To'] = weechat.config_get_plugin('email_to')
    msg['Subject'] = subject

    p = subprocess.Popen(
        [weechat.config_get_plugin('sendmail'),
         weechat.config_get_plugin('email_to')],
        stdin=subprocess.PIPE,

    )
    p.communicate(input=str(msg))


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------
def main():
    '''Sets up WeeChat notifications.'''
    # Initialize options.
    for option, value in SETTINGS.items():
        if not weechat.config_is_set_plugin(option):
            weechat.config_set_plugin(option, value)
    # Initialize.
    notifications = [
        'Public',
        'Private',
        'Action',
        'Notice',
        'Invite',
        'Highlight',
        'Server',
        'Channel',
        'DCC',
        'WeeChat'
    ]
    # Register hooks.
    weechat.hook_signal(
        'irc_server_connected',
        'cb_irc_server_connected',
        '')
    weechat.hook_signal(
        'irc_server_disconnected',
        'cb_irc_server_disconnected',
        '')
    weechat.hook_signal('upgrade_ended', 'cb_upgrade_ended', '')
    weechat.hook_print('', '', '', 1, 'cb_process_message', '')


if __name__ == '__main__' and weechat.register(
    SCRIPT_NAME,
    SCRIPT_AUTHOR,
    SCRIPT_VERSION,
    SCRIPT_LICENSE,
    SCRIPT_DESC,
    '',
    ''
):
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 by Filip H.F. "FiXato" Slagter <fixato+weechat@gmail.com>
#
# Shutup: a quick WeeChat script to replace text from specified users with
#         random or preset text as a way to hide their actual text.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# 2014-01-31: FiXato, (freenode.#weechat)
#       0.1 : initial release
#
# requires: WeeChat version 0.3.6 or higher
#
# Thanks go out to nils_2 for providing his skeleton.py template, available at https://github.com/weechatter/weechat-scripts/blob/master/python/skeleton.py
#
# Development is currently hosted at
# https://github.com/FiXato/weechat_scripts

try:
  import weechat,re
  from random import choice

except Exception:
  print "This script must be run under WeeChat."
  print "Get WeeChat now at: http://www.weechat.org/"
  quit()

SCRIPT_NAME     = "shutup"
SCRIPT_AUTHOR   = 'Filip H.F. "FiXato" Slagter <FiXato+weechat@gmail.com>'
SCRIPT_VERSION  = "0.2"
SCRIPT_LICENSE  = "GPL"
SCRIPT_DESC     = "Replace text from specified IRC users with random or preset text as a way to hide their actual text. Unlike /filter it won't hide the line (and thus can't be toggled either), and has access to the entire hostmask for comparison. Can be useful to mute users while still seeing that they are active."

OPTIONS         = {
                    'replacement_text'        : ('','Replacement text for everything the muted user says. Leave empty to use random lines from the Jabberwocky poem.'),
                    'muted_masks'             : ('','Space-separated regular expressions that will be matched against the nick!ident@host.mask. Any user matching will get their message muted. Can also include a comma-separated list of channels for every regular expression separated from the regexp by a colon. Prefix regexp with (?i) if you want it to be case insensitive. Example: "@\S+\.aol\.com$:#comcast,#AT&T (?i)!root@\S+" would mute messages in channels #comcast and #AT&T from users whose hosts end in *.aol.com, as well as all users who have any case variation of root as ident regardless of channel.'),
                  }
DEBUG = False
jabberwocky = """
'Twas brillig, and the slithy toves
Did gyre and gimble in the wabe;
All mimsy were the borogoves,
And the mome raths outgrabe.

"Beware the Jabberwock, my son!
The jaws that bite, the claws that catch!
Beware the Jubjub bird, and shun
The frumious Bandersnatch!"

He took his vorpal sword in hand:
Long time the manxome foe he sought—
So rested he by the Tumtum tree,
And stood awhile in thought.

And as in uffish thought he stood,
The Jabberwock, with eyes of flame,
Came whiffling through the tulgey wood,
And burbled as it came!

One, two! One, two! and through and through
The vorpal blade went snicker-snack!
He left it dead, and with its head
He went galumphing back.

"And hast thou slain the Jabberwock?
Come to my arms, my beamish boy!
O frabjous day! Callooh! Callay!"
He chortled in his joy.

'Twas brillig, and the slithy toves
Did gyre and gimble in the wabe;
All mimsy were the borogoves,
And the mome raths outgrabe.
"""
replacement_lines = filter(None, jabberwocky.splitlines())

def random_replacement_line(lines = replacement_lines):
  return choice(lines)

def replacement_line():
  global OPTIONS
  if OPTIONS['replacement_text'] == '':
    return random_replacement_line()
  return OPTIONS['replacement_text']

# Easily use weechat colors in the script
#   text = substitute_colors('my text ${color:yellow}yellow${color:default} colored.')
# eval_expression():  to match ${color:nn} tags
regex_color=re.compile('\$\{color:([^\{\}]+)\}')
def substitute_colors(text):
  if int(version) >= 0x00040200:
    return weechat.string_eval_expression(text,{},{},{})
  # substitute colors in output
  return re.sub(regex_color, lambda match: weechat.color(match.group(1)), text)

# ===================[ weechat options & description ]===================
def init_options():
  for option,value in OPTIONS.items():
    if not weechat.config_is_set_plugin(option):
      weechat.config_set_plugin(option, value[0])
      toggle_refresh(None, 'plugins.var.python.' + SCRIPT_NAME + '.' + option, value[0])
    else:
      toggle_refresh(None, 'plugins.var.python.' + SCRIPT_NAME + '.' + option, weechat.config_get_plugin(option))
    weechat.config_set_desc_plugin(option, '%s (default: "%s")' % (value[1], value[0]))

def debug(str):
  if DEBUG:
    weechat.prnt("", str)

def update_muted_masks(masks):
  global muted_masks
  muted_masks = {}
  for mask in masks.split():
    if '#' in mask:
      mask, chan = mask.split(':',1)
      channels = [channel.lower() for channel in chan.split(',')]
    else:
      channels = []
    muted_masks[mask] = [re.compile(mask), channels]
  debug('muted masks: %s' % muted_masks)

def toggle_refresh(pointer, name, value):
  global OPTIONS
  option = name[len('plugins.var.python.' + SCRIPT_NAME + '.'):]        # get optionname
  OPTIONS[option] = value                                               # save new value
  if option == 'muted_masks':
    update_muted_masks(value)
  return weechat.WEECHAT_RC_OK

def shutup_cb(data, modifier, modifier_data, string):
  dict_in = { "message": string }
  message_ht = weechat.info_get_hashtable("irc_message_parse", dict_in)

  hostmask = message_ht['host']
  arguments = message_ht['arguments']
  channel = message_ht['channel']

  new_arguments = re.sub(r'^%s :.+' % channel, lambda x: '%s :%s' % (channel, replacement_line()), arguments)
  new_string = re.sub(r'%s$' % re.escape(arguments), lambda x: new_arguments, string)

  for key, [mask_regexp, channels] in muted_masks.iteritems():
    # If there is one or more channels listed for this mask regexp, and none of them match the current channel, continue to the next mute mask
    if len(channels) > 0 and channel.lower() not in channels:
      debug("%s doesn't match any of the listed channels: %s" % (channel, channels))
      continue

    # If the hostmask matches the mask regular expression, return the new, manipulated, string.
    debug("comparing %s to %s" % (mask_regexp.pattern, hostmask))
    if mask_regexp.search(hostmask):
      debug("  %s matches %s" % (mask_regexp.pattern, hostmask))
      return new_string
  # Nothing matches, so return the original, unmodified, string
  return string

# ================================[ main ]===============================
if __name__ == "__main__":
  if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
    version = weechat.info_get("version_number", "") or 0

    if int(version) >= 0x00030600:
      # init options from your script
      init_options()
      # create a hook for your options
      weechat.hook_config( 'plugins.var.python.' + SCRIPT_NAME + '.*', 'toggle_refresh', '' )
    else:
      weechat.prnt("","%s%s %s" % (weechat.prefix("error"),SCRIPT_NAME,": needs version 0.3.6 or higher"))
      weechat.command("","/wait 1ms /python unload %s" % SCRIPT_NAME)

    hook = weechat.hook_modifier("irc_in_privmsg", "shutup_cb", "")



# -*- coding: utf-8 -*-
# MIT License
#
# Copyright (c) 2017 Matthias Adamczyk
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""
Automatically hide read IRC buffers and unhide them on new activity.

Requires WeeChat version 1.0 or higher.

Configuration:
    plugins.var.python.buffer_autohide.hide_inactive: Hide inactive buffers (default: "off")
    plugins.var.python.buffer_autohide.hide_private: Hide private buffers (default: "off")
    plugins.var.python.buffer_autohide.unhide_low: Unhide a buffer when a low message (like JOIN,
        PART, etc.) has been received (default: "off")

History:
2017-03-19: Matthias Adamczyk <mail@notmatti.me>
    version 0.1: Initial release

https://github.com/notmatti/buffer_autohide
"""
import_ok = True
try:
    import weechat
    from weechat import WEECHAT_RC_OK
except ImportError:
    print("Script must be run under weechat. https://weechat.org")
    import_ok = False

SCRIPT_NAME = "buffer_autohide"
SCRIPT_AUTHOR = "Matthias Adamczyk <mail@notmatti.me>"
SCRIPT_VERSION = "0.1"
SCRIPT_LICENSE = "MIT"
SCRIPT_DESC = "Automatically hide read IRC buffers and unhide them on new activity"
SCRIPT_COMMAND = SCRIPT_NAME

CURRENT_BUFFER = "0x0" # pointer string representation


def config_init():
    """Add configuration options to weechat."""
    config = {
        "hide_inactive": ("off", "Hide inactive buffers"),
        "hide_private": ("off", "Hide private buffers"),
        "unhide_low": ("off",
            "Unhide a buffer when a low message (like JOIN, PART, etc.) has been received"),
    }

    for option, default_value in config.items():
        if weechat.config_get_plugin(option) == "":
            weechat.config_set_plugin(option, default_value[0])
        weechat.config_set_desc_plugin(
            option, '{} (default: "{}")'.format(default_value[1], default_value[0]))


def hotlist_dict():
    """Return the contents of the hotlist as a dictionary.

    The returned dictionary has the following structure:
    >>> hotlist = {
    ...     "0x0": {                    # string representation of the buffer pointer
    ...         "count_low": 0,
    ...         "count_message": 0,
    ...         "count_private": 0,
    ...         "count_highlight": 0,
    ...     }
    ... }
    """
    hotlist = {}
    infolist = weechat.infolist_get("hotlist", "", "")
    while weechat.infolist_next(infolist):
        buffer_pointer = weechat.infolist_pointer(infolist, "buffer_pointer")
        hotlist[buffer_pointer] = {}
        hotlist[buffer_pointer]["count_low"] = weechat.infolist_integer(
            infolist, "count_00")
        hotlist[buffer_pointer]["count_message"] = weechat.infolist_integer(
            infolist, "count_01")
        hotlist[buffer_pointer]["count_private"] = weechat.infolist_integer(
            infolist, "count_02")
        hotlist[buffer_pointer]["count_highlight"] = weechat.infolist_integer(
            infolist, "count_03")
    weechat.infolist_free(infolist)
    return hotlist


def hide_buffer_cb(signal, data, signal_data):
    """Hide the previous IRC buffer when switching buffers.

    If configuration option ``hide_private`` is enabled,
    private buffers will be hidden as well.
    """
    global CURRENT_BUFFER

    previous_buffer = CURRENT_BUFFER
    CURRENT_BUFFER = weechat.current_buffer()

    plugin = weechat.buffer_get_string(previous_buffer, "plugin")
    full_name = weechat.buffer_get_string(previous_buffer, "full_name")
    server = weechat.buffer_get_string(previous_buffer, "localvar_server")
    channel = weechat.buffer_get_string(previous_buffer, "localvar_channel")

    if plugin != "irc" or full_name.startswith("irc.server"):
        return WEECHAT_RC_OK

    buffer_type = weechat.buffer_get_string(
        weechat.info_get("irc_buffer", "{},{}".format(server, channel)),
        "localvar_type")

    if (buffer_type == "private"
            and weechat.config_get_plugin("hide_private") == "off"):
        return WEECHAT_RC_OK

    if weechat.config_get_plugin("hide_inactive") == "off":
        nicks_count = 0
        infolist = weechat.infolist_get(
            "irc_channel", "", "{},{}".format(server, channel))
        if infolist:
            weechat.infolist_next(infolist)
            nicks_count = weechat.infolist_integer(infolist, "nicks_count")
        weechat.infolist_free(infolist)
        if nicks_count == 0:
            return WEECHAT_RC_OK

    weechat.buffer_set(previous_buffer, "hidden", "1")
    return WEECHAT_RC_OK


def unhide_buffer_cb(data, signal, signal_data):
    """Unhide a buffer on new activity.

    This callback unhides the buffer in which a new message has been received.
    If configuration option ``unhide_low`` is enabled,
    buffers with only low messages (like JOIN, PART, etc.) will be unhidden as well.
    """
    server = signal.split(",")[0]
    message = weechat.info_get_hashtable(
        "irc_message_parse",
        {"message": signal_data})
    channel = message["channel"]
    hotlist = hotlist_dict()
    buffer = weechat.info_get("irc_buffer", "{},{}".format(server, channel))

    if not buffer in hotlist.keys():
        # just some background noise
        return WEECHAT_RC_OK

    if (weechat.config_get_plugin("unhide_low") == "on"
            and hotlist[buffer]["count_low"] > 0
            or hotlist[buffer]["count_message"] > 0
            or hotlist[buffer]["count_private"] > 0
            or hotlist[buffer]["count_highlight"] > 0):
        weechat.buffer_set(buffer, "hidden", "0")

    return WEECHAT_RC_OK


if (__name__ == '__main__'
        and import_ok
        and weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                             SCRIPT_DESC, '', '')):
    weechat_version = weechat.info_get("version_number", "") or 0
    if int(weechat_version) >= 0x01000000:
        config_init()
        CURRENT_BUFFER = weechat.current_buffer()
        weechat.hook_signal("buffer_switch", "hide_buffer_cb", "")
        weechat.hook_signal("*,irc_in2_*", "unhide_buffer_cb", "")
    else:
        weechat.prnt("", "{}{} requires WeeChat version 1.0 or higher".format(
            weechat.prefix('error'), SCRIPT_NAME))



# Show differences between old and new topics
# Author: Dafydd Harries <daf@rhydd.org>
# License: GPL3

import re

from itertools import izip_longest

import weechat

SCRIPT_NAME    = "topicdiff"
SCRIPT_AUTHOR  = "Dafydd Harries <daf@rhydd.org>"
SCRIPT_VERSION = "0.3"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Show differences between old and new topics."

pending_change_buffer = None
topics = {}

def topic_chunks(s):
    return re.split(r'\s+[-~|]+\s+', s)

def topic_changed(buffer, new_topic):
    if buffer in topics:
        old_chunks = topic_chunks(topics[buffer])
        new_chunks = topic_chunks(new_topic)

        for old_chunk, new_chunk in izip_longest(old_chunks, new_chunks):
            if old_chunk and old_chunk not in new_chunks:
                weechat.prnt(buffer, '%s-\t%s' % (
                    weechat.color('red'),
                    old_chunk
                ))

            if new_chunk and new_chunk not in old_chunks:
                weechat.prnt(buffer, '%s+\t%s' % (
                    weechat.color('green'),
                    new_chunk
                ))

    topics[buffer] = new_topic

def print_332(data, buffer, time, tags, displayed, highlight, prefix, message):
    #weechat.prnt('', 'print: %r' % (a,))
    global pending_change_buffer
    pending_change_buffer = buffer
    return weechat.WEECHAT_RC_OK

def print_topic(
        data, buffer, time, tags, displayed, highlight, prefix, message):
    global pending_change_buffer
    pending_change_buffer = buffer
    return weechat.WEECHAT_RC_OK

def irc_in2_332(data, tags, msg):
    global pending_change_buffer

    if pending_change_buffer is None:
        # Hmm, that's weird.
        #weechat.prnt('', 'no pending buffer :(')
        return weechat.WEECHAT_RC_OK

    match = re.match(r':\S+ 332 \S+ \S+ :(.*)', msg)

    if not match:
        #weechat.prnt('', 'no match :(')
        return weechat.WEECHAT_RC_OK

    new_topic = match.group(1)
    topic_changed(pending_change_buffer, new_topic)
    pending_change_buffer = None
    return weechat.WEECHAT_RC_OK

def irc_in2_topic(data, tags, msg):
    global pending_change_buffer

    #weechat.prnt('', '%r' % ((tags, msg),))

    if pending_change_buffer is None:
        # Hmm, that's weird.
        #weechat.prnt('', 'no pending buffer :(')
        return weechat.WEECHAT_RC_OK

    match = re.match(r':\S+ TOPIC \S+ :(.*)', msg)

    if not match:
        #weechat.prnt('', 'no match :(')
        return weechat.WEECHAT_RC_OK

    new_topic = match.group(1)
    topic_changed(pending_change_buffer, new_topic)
    pending_change_buffer = None
    return weechat.WEECHAT_RC_OK

def register():
    weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
        SCRIPT_LICENSE, SCRIPT_DESC, '', '')

    weechat.hook_print('', 'irc_332', '', 1, 'print_332', '')
    weechat.hook_print('', 'irc_topic', '', 1, 'print_topic', '')
    weechat.hook_signal('*,irc_in2_332', 'irc_in2_332', '')
    weechat.hook_signal('*,irc_in2_topic', 'irc_in2_topic', '')

if __name__ == '__main__':
    register()




# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2015 by nils_2 <weechatter@arcor.de>
#
# save channel key from protected channel(s) to autojoin or secure data
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# idea by freenode.elsae
#
# 2015-05-09: nils_2, (freenode.#weechat)
#       0.3 : fix: ValueError (reported by: Darpa)
#
# 2014-12-20: nils_2, (freenode.#weechat)
#       0.2 : add option "add" to automatically add channel/key to autojoin option after a /join (idea by Prezident)
#
# 2013-10-03: nils_2, (freenode.#weechat)
#       0.1 : initial release
#
# requires: WeeChat version 0.3.2
#
# Development is currently hosted at
# https://github.com/weechatter/weechat-scripts

try:
    import weechat,re

except Exception:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    quit()

SCRIPT_NAME     = "autosavekey"
SCRIPT_AUTHOR   = "nils_2 <weechatter@arcor.de>"
SCRIPT_VERSION  = "0.3"
SCRIPT_LICENSE  = "GPL"
SCRIPT_DESC     = "save channel key from protected channel(s) to autojoin or secure data"

OPTIONS         = { 'mute'        : ('off','execute command silently, only error messages will be displayed.'),
                    'secure'      : ('off','change channel key in secure data.'),
                    'add'         : ('off','adds channel and key to autojoin list on /join, if channel/key does not already exists'),
                  }
# /join #channel key
# signal = freenode,irc_raw_in_324
# signal_data = :asimov.freenode.net 324 nick #channel +modes key
def irc_raw_in_324_cb(data, signal, signal_data):
    parsed = get_hashtable(signal_data)
    server = signal.split(',',1)[0]
    argv = parsed['arguments'].split(" ")

    # buffer without channel key
    if len(argv) < 4:
        return weechat.WEECHAT_RC_OK

    channel = argv[1]
    new_key = argv[3]

    autojoin_list = get_autojoin(server)
    if not autojoin_list:
        return weechat.WEECHAT_RC_OK

    # check autojoin for space
    if len(re.findall(r" ", autojoin_list)) > 1:
        weechat.prnt('', '%s%s: autojoin format for server "%s" invalid (two or more spaces).' % (weechat.prefix('error'),SCRIPT_NAME,server) )
        return weechat.WEECHAT_RC_OK

    # no keylist, only channels in autojoin option
    if len(re.findall(r" ", autojoin_list)) == 0:
        argv_channels = autojoin_list.split(',')
        argv_keys = []
    else:
        # split autojoin option to a channel and a key list
        arg_channel,arg_keys = autojoin_list.split(' ')
        argv_channels = arg_channel.split(',')
        argv_keys = arg_keys.split(',')

    # check channel position
    try:
        channel_position = argv_channels.index(channel)
    except ValueError:
        channel_position = -1

    sec_data = 0
    # does buffer already exist in autojoin list?
    if channel_position >= 0:
        # remove channel from list
        argv_channels.pop(channel_position)
        # check if there is at least one key in list
        if len(argv_keys) >= 1:
            # check channel position and number of keys
            if channel_position <= len(argv_keys):
                # remove key from list
                sec_data = check_key_for_secure(argv_keys,channel_position)
                sec_data_name = argv_keys[channel_position][11:-1]
                argv_keys.pop(channel_position)
    else:
        if OPTIONS['add'].lower() == 'off':
            return weechat.WEECHAT_RC_OK

    # add channel and key at first position
    argv_channels.insert(0, channel)
    argv_keys.insert(0,new_key)


    # check weechat version and if secure option is on and secure data will be used for this key?
    if int(version) >= 0x00040200 and OPTIONS['secure'].lower() == 'on' and sec_data == 1:
        weechat.command('','%s/secure set %s %s' % (use_mute(),sec_data_name,new_key))
    else:
        if sec_data == 1:
            weechat.prnt('', '%s%s: key for channel "%s.%s" not changed! option "plugins.var.python.%s.secure" is off and you are using secured data for key.' % (weechat.prefix('error'),SCRIPT_NAME,server,channel,SCRIPT_NAME) )
            return weechat.WEECHAT_RC_OK
        new_joined_option = '%s %s' % (','.join(argv_channels),','.join(argv_keys))
        save_autojoin_option(server,new_joined_option)
    return weechat.WEECHAT_RC_OK

# replace an already existing channel key with an new one
# when OP changes channel key
def irc_raw_in_mode_cb(data, signal, signal_data):
    parsed = get_hashtable(signal_data)

    server = signal.split(',',1)[0]
    argv = parsed['arguments'].split(" ")

    if argv[1] != "+k":
        return weechat.WEECHAT_RC_OK

    channel = argv[0]
    new_key = argv[2]

    add_key_to_list(server,channel,new_key)
    return weechat.WEECHAT_RC_OK

def add_key_to_list(server,channel,new_key):
    autojoin_list = get_autojoin(server)
    if not autojoin_list:
        return weechat.WEECHAT_RC_OK

    # check autojoin for space
    if len(re.findall(r" ", autojoin_list)) == 0:
        weechat.prnt('', '%s%s: no password(s) set in autojoin for server "%s".' % (weechat.prefix('error'),SCRIPT_NAME,server) )
        return weechat.WEECHAT_RC_OK
    if len(re.findall(r" ", autojoin_list)) > 1:
        weechat.prnt('', '%s%s: autojoin format for server "%s" invalid (two or more spaces).' % (weechat.prefix('error'),SCRIPT_NAME,server) )
        return weechat.WEECHAT_RC_OK


    # split autojoin option to a channel and a key list
    arg_channel,arg_keys = autojoin_list.split(' ')
    argv_channels = arg_channel.split(',')
    argv_keys = arg_keys.split(',')

    # search for channel name in list of channels and get position
    if channel in argv_channels:
        channel_pos_in_list = argv_channels.index(channel)
        # enough keys in list? list counts from 0!
        if channel_pos_in_list + 1 > len(argv_keys):
            weechat.prnt('', '%s%s: not enough keys in list or channel position is not valid. check out autojoin option for server "%s".' % (weechat.prefix('error'),SCRIPT_NAME,server) )
            return weechat.WEECHAT_RC_OK

        sec_data = check_key_for_secure(argv_keys,channel_pos_in_list)

        # check weechat version and if secure option is on and secure data will be used for this key?
        if int(version) >= 0x00040200 and OPTIONS['secure'].lower() == 'on' and sec_data == 1:
            sec_data_name = argv_keys[channel_pos_in_list][11:-1]
            weechat.command('','%s/secure set %s %s' % (use_mute(),sec_data_name,new_key))
        else:
            if sec_data == 1:
                weechat.prnt('', '%s%s: key for channel "%s.%s" not changed! option "plugins.var.python.%s.secure" is off and you are using secured data for key.' % (weechat.prefix('error'),SCRIPT_NAME,server,channel,SCRIPT_NAME) )
                return weechat.WEECHAT_RC_OK
            argv_keys[channel_pos_in_list] = new_key
            new_joined_option = '%s %s' % (','.join(argv_channels),','.join(argv_keys))
            save_autojoin_option(server,new_joined_option)
    return weechat.WEECHAT_RC_OK

def get_hashtable(string):
    parsed = weechat.info_get_hashtable('irc_message_parse', dict(message=string))
    try:
        parsed['message'] = parsed['arguments'].split(' :', 1)[1]
    except:
        parsed['message'] = ""
    return parsed

def get_autojoin(server):
    return weechat.config_string(weechat.config_get('irc.server.%s.autojoin' % server))

def find_element_in_list(element,list_element):
        try:
            index_element=list_element.index(element)
            return index_element
        except ValueError:
            return -1

def save_autojoin_option(server,new_joined_option):
    weechat.command('','%s/set irc.server.%s.autojoin %s' % (use_mute(),server,new_joined_option))

def use_mute():
    use_mute = ''
    if OPTIONS['mute'].lower() == 'on':
        use_mute = '/mute '
    return use_mute

# check key for "${sec.data."
def check_key_for_secure(argv_keys,position):
    sec_data = 0
    if argv_keys[position][0:11] == '${sec.data.':
        sec_data = 1
    return sec_data
# ================================[ weechat options & description ]===============================
def init_options():
    for option,value in OPTIONS.items():
        if not weechat.config_is_set_plugin(option):
            weechat.config_set_plugin(option, value[0])
            OPTIONS[option] = value[0]
        else:
            OPTIONS[option] = weechat.config_get_plugin(option)
        weechat.config_set_desc_plugin(option, '%s (default: "%s")' % (value[1], value[0]))

def toggle_refresh(pointer, name, value):
    global OPTIONS
    option = name[len('plugins.var.python.' + SCRIPT_NAME + '.'):]        # get optionname
    OPTIONS[option] = value                                               # save new value
    return weechat.WEECHAT_RC_OK

# ================================[ main ]===============================
if __name__ == "__main__":
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        version = weechat.info_get("version_number", "") or 0

        if int(version) >= 0x00030200:
            init_options()
            weechat.hook_config( 'plugins.var.python.' + SCRIPT_NAME + '.*', 'toggle_refresh', '' )
            weechat.hook_signal("*,irc_raw_in_mode","irc_raw_in_mode_cb","")
            weechat.hook_signal("*,irc_raw_in_324","irc_raw_in_324_cb","")
        else:
            weechat.prnt("","%s%s %s" % (weechat.prefix("error"),SCRIPT_NAME,": needs version 0.3.2 or higher"))
            weechat.command("","/wait 1ms /python unload %s" % SCRIPT_NAME)



#
# Copyright (c) 2010, Chris Branch <x@chrisbranch.co.uk>
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the <organization> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# History:
# 2010-07-26
#   revision 1.0
#

import weechat
import time

SCRIPT_NAME    = "ichatts"
SCRIPT_AUTHOR  = "Chris Branch <x@chrisbranch.co.uk>"
SCRIPT_VERSION = "1.0"
SCRIPT_LICENSE = "BSD"
SCRIPT_DESC    = "iChat-style timestamps"

settings = {
    "minutes_until_timestamp" : '5',    # print when no message occurs for this long
    "remind_every"            : '15',   # print a new timestamp every X minutes if there is activity
}

buffer_dates = {}

def prnt_timestamp(buffer, timestamp):
    weechat.prnt(buffer, '%s[%s%s%s:%s%s%s]' %
	(weechat.color("chat_delimiters"),
	 weechat.color("chat_time"),
	 time.strftime('%H', time.localtime(timestamp)),
	 weechat.color("chat_time_delimiters"),
	 weechat.color("chat_time"),
	 time.strftime('%M', time.localtime(timestamp)),
	 weechat.color("chat_delimiters")))

def timer_cb(data, remaining_calls):
    global buffer_dates
    current_time = int(time.time())
    timestamp_secs = int(weechat.config_get_plugin('minutes_until_timestamp')) * 60
    # Which buffers need a timestamp printing?
    for (buffer, (last_message, last_printed)) in buffer_dates.items():
        # If X minutes have elapsed since the last message, and we haven't printed anything since then.
        if last_printed < last_message and current_time - last_message >= timestamp_secs:
            buffer_dates[buffer] = (last_message, current_time)
            prnt_timestamp(buffer, last_message)
    return weechat.WEECHAT_RC_OK

def print_cb(data, buffer, date, tags, displayed, highlight, prefix, message):
    # Update buffer with date of last message
    global buffer_dates
    current_time = int(date)
    last_printed = buffer_dates.get(buffer, (0, 0))[1]
    remind_secs = int(weechat.config_get_plugin('remind_every')) * 60
    # Has it been X minutes since we last printed a timestamp?
    if current_time - last_printed >= remind_secs:
        last_printed = current_time
        prnt_timestamp(buffer, current_time)
        
    buffer_dates[buffer] = (current_time, last_printed)
    return weechat.WEECHAT_RC_OK

if __name__ == "__main__":
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                        SCRIPT_DESC, "", ""):
        # Set default settings
        for option, default_value in settings.iteritems():
            if not weechat.config_is_set_plugin(option):
                weechat.config_set_plugin(option, default_value)

	weechat.hook_timer(60000, 60, 0, 'timer_cb', '')
        weechat.hook_print('', '', '', 0, 'print_cb', '')




# -*- coding: utf-8 -*-
# Author: Caspar Clemens Mierau <ccm@screenage.de>
# Homepage: https://github.com/leitmedium/weechat-irssinotifier
# Derived from: notifo
#   Author: ochameau <poirot.alex AT gmail DOT com>
#   Homepage: https://github.com/ochameau/weechat-notifo
# And from: notify
#   Author: lavaramano <lavaramano AT gmail DOT com>
#   Improved by: BaSh - <bash.lnx AT gmail DOT com>
#   Ported to Weechat 0.3.0 by: Sharn - <sharntehnub AT gmail DOT com)
# And from: notifo_notify
#   Author: SAEKI Yoshiyasu <laclef_yoshiyasu@yahoo.co.jp>
#   Homepage: http://bitbucket.org/laclefyoshi/weechat/
#
# This plugin brings IrssiNotifier to your Weechat. Setup and install
# IrssiNotifier first: https://irssinotifier.appspot.com
#
# Requires Weechat >= 0.3.7, openssl
# Released under GNU GPL v3
#
# 2017-05-17, das_aug <wct@fnanp.in-ulm.de>
#     version 0.8.1 - change openssl commandline to how the android app uses it now
#                     (add "-md md5")
# 2017-05-11, paalka <paal@128.no>
#     version 0.8: - add the ability to store the API token and
#                    encryption key as secured data.
# 2016-01-11, dbendit <david@ibendit.com>
#     version 0.7: - ignore_nicks option
# 2014-05-10, Sébastien Helleu <flashcode@flashtux.org>
#     version 0.6.3: - change hook_print callback argument type of
#                      displayed/highlight (WeeChat >= 1.0)
# 2013-12-07, zigdon
#     version 0.6.2: - support ignoring all buffers in a server, add help text.
# 2013-08-20, balu
#     version 0.6.1: - support for every private notification not only irc (especialy also jabber)
# 2013-08-16, kang@insecure.ws
#     version 0.6: - only_away option (only notify if set away)
# 2013-01-18, ccm <ccm@screenage.de>:
#     version 0.5: - removed version check and legacy curl usage
# 2012-12-27, ccm <ccm@screenage.de>:
#     version 0.4: - use non-blocking hook_process_hashtable for url call
#                    for weechat >= 0.3.7
# 2012-12-22, ccm <ccm@screenage.de>:
#     version 0.3: - no longer notifies if the message comes from the user
#                    itself
#                  - removed curl dependency
#                  - cleaned up openssl call
#                  - no more crashes due to missing escaping
#                  - Kudos to Juergen "@tante" Geuter <tante@the-gay-bar.com>
#                    for the patches!
# 2012-10-27, ccm <ccm@screenage.de>:
#     version 0.2: - curl uses secure command call (decreases risk of command
#                    injection)
#                  - correct split of nick and channel name in a hilight
# 2012-10-26, ccm <ccm@screenage.de>:
#     version 0.1: - initial release - working proof of concept

import weechat, string, os, urllib, urllib2, shlex
from subprocess import Popen, PIPE

weechat.register("irssinotifier",
                 "Caspar Clemens Mierau <ccm@screenage.de>",
                 "0.8.1",
                 "GPL3",
                 "irssinotifier: Send push notifications to Android's IrssiNotifier about your private message and highligts.",
                 "",
                 "")

settings = {
    "api_token": "API token from http://irssinotifier.appspot.com.",
    "encryption_password": "Your password, same as on the phone's client.",
    "only_away": "Only send notifications when set as away.",
    "ignore_buffers": "Comma separated list of buffers to ignore.",
    "ignore_servers": "Comma separated list of servers to ignore.",
    "ignore_nicks": "Comma separated list of nicks to ignore.",
}

required_settings = ["api_token", "encryption_password"]

for option, help_text in settings.items():
    if not weechat.config_is_set_plugin(option):
        weechat.config_set_plugin(option, "")

    if option in required_settings and weechat.config_get_plugin(option) == "":
        weechat.prnt("", weechat.prefix("error") + "irssinotifier: Please set option: %s" % option)
        weechat.prnt("", "irssinotifier: /set plugins.var.python.irssinotifier.%s STRING" % option)

    weechat.config_set_desc_plugin(option, help_text)

# Hook privmsg/hilights
weechat.hook_print("", "notify_message", "", 1, "notify_show", "")
weechat.hook_print("", "notify_private", "", 1, "notify_show", "")

# Functions
def notify_show(data, bufferp, uber_empty, tagsn, isdisplayed,
        ishilight, prefix, message):

    # irc PMs are caught by notify_private, but we need notify_message to
    # capture hilights in channels.
    if 'notify_message' in tagsn and not ishilight:
        return weechat.WEECHAT_RC_OK

    # are we away?
    away = weechat.buffer_get_string(bufferp,"localvar_away")
    if (away == "" and weechat.config_get_plugin("only_away") == "on"):
        return weechat.WEECHAT_RC_OK

    # get local nick for buffer
    mynick = weechat.buffer_get_string(bufferp,"localvar_nick")

    # get buffer info
    name = weechat.buffer_get_string(bufferp,"name")
    server = weechat.buffer_get_string(bufferp, "localvar_server")
    channel = weechat.buffer_get_string(bufferp, "localvar_channel")

    # ignore buffers on ignorelists
    if not (server in weechat.config_get_plugin("ignore_servers").split(",") or
        name in weechat.config_get_plugin("ignore_buffers").split(",") or
        prefix in weechat.config_get_plugin("ignore_nicks").split(",")):

        # only notify if the message was not sent by myself
        if (weechat.buffer_get_string(bufferp, "localvar_type") == "private") and (prefix!=mynick):
            show_notification(channel, prefix, message)

        elif int(ishilight):
            buffer = (weechat.buffer_get_string(bufferp, "short_name") or name)
            show_notification(buffer, prefix, message)

    return weechat.WEECHAT_RC_OK

def encrypt(text):
    encryption_password = weechat.config_get_plugin("encryption_password")

    # decrypt the password if it is stored as secured data
    if encryption_password.startswith("${sec."):
        encryption_password = weechat.string_eval_expression(encryption_password, {}, {}, {})

    command="openssl enc -aes-128-cbc -salt -base64 -md md5 -A -pass env:OpenSSLEncPW"
    opensslenv = os.environ.copy();
    opensslenv['OpenSSLEncPW'] = encryption_password
    output,errors = Popen(shlex.split(command),stdin=PIPE,stdout=PIPE,stderr=PIPE,env=opensslenv).communicate(text+" ")
    output = string.replace(output,"/","_")
    output = string.replace(output,"+","-")
    output = string.replace(output,"=","")
    return output

def show_notification(chan, nick, message):
    API_TOKEN = weechat.config_get_plugin("api_token")

    # decrypt the API token if it is stored as secured data
    if API_TOKEN.startswith("${sec."):
        API_TOKEN = weechat.string_eval_expression(API_TOKEN, {}, {}, {})

    if API_TOKEN != "":
        url = "https://irssinotifier.appspot.com/API/Message"
        postdata = urllib.urlencode({'apiToken':API_TOKEN,'nick':encrypt(nick),'channel':encrypt(chan),'message':encrypt(message),'version':13})
        version = weechat.info_get("version_number", "") or 0
        hook1 = weechat.hook_process_hashtable("url:"+url, { "postfields":  postdata}, 2000, "", "")

# vim: autoindent expandtab smarttab shiftwidth=4



# Copyright (c) 2010 by Stephan Huebner <s.huebnerfun01@gmx.org>
#
# Intended use:
#
# play a soundfile when a message comes in (for choosable buffers)
#
# TODO: implement playing a soundfile when a highlight occurs
# TODO: (possibly) show the number of unread private messages and
#       highlights in status bar
# TODO: Find out, why sometimes "w.hook_process" doesn't seem to
#       start (or at least no sound is being played)
# TODO: (possibly) Change app-choosing to some predefined options (like
#       mplayer, vlc or ogg123, etc...)
# TODO: Make the whole script more Python-like (if I find out how...)
# TODO: Make the script not beep on message from yourself
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# History:
#	2010-10-17: version 0.1: initial release
#
#  2010-10-17: version 0.1.1:
#     * change: If a buffer + sound is added and psound is empty, then
#                psound is set to given sound
#     * fix: bugs in "on"- and "off"-functions
#     * fix: "on"- and "off"-statusses were not used
#  2010-11-15: Version 0.1.3
#		* change: No sound is played anymore on messages from yourself.
#		* fix: a new sound for an entry already in the list wasn't set.
#		* change: Replaced "weechat.hook_process" with Pythons' "subprocess.Popen"

import subprocess as s
from os.path import expanduser

SCR_NAME    = "snotify"
SCR_AUTHOR  = "Stephan Huebner <shuebnerfun01@gmx.org>"
SCR_VERSION = "0.1.3"
SCR_LICENSE = "GPL3"
SCR_DESC    = "Play a soundfile for messages in choosable channels or queries"
SCR_COMMAND = "snotify"

import_ok = True

bfList = []
muted = False

try:
	import weechat as w
except:
	print "Script must be run under weechat. http://www.weechat.org"
	import_ok = False

settings = {
	"player" : "mplayer", # Application used to play a soundfile
	"psound" : "", # Sound that should be played on private messages
	"hsound" : "", # Sound that should be played on highlights
	"buffers" : ""
}

def errMsg(myMsg):
	alert("ERR: " + myMsg)
	return

def fn_hook_process(data, command, rc, stdout, stderr):
	alert(stderr)
	return w.WEECHAT_RC_OK
	
def fn_privmsg(data, bufferp, time, tags, display, is_hilight, prefix, msg):
	global bfList
	global settings
	servername = (w.buffer_get_string(bufferp, "name").split("."))[0]
	ownNick = w.info_get("irc_nick", servername)
	mySound = ""
	if not muted and prefix != ownNick:
		if settings["player"] == "":
			errMsg("'Player' isn't set!")
		else:
			for lEntry in bfList:
				if lEntry["buffer"] == w.buffer_get_string(bufferp, "name"):
					# we found a buffer of that name
					if lEntry["status"] == "on":
						if lEntry["sound"] == "":
							if settings["psound"] == "":
								errMsg("No sound defined! Please set either the " +
										 "regular 'psound'-option or the 'sound'-" +
										 "option for this buffer.")
							else:
								mySound = settings["psound"]
						else:
							mySound = lEntry["sound"]
						s.Popen([settings["player"], expanduser(mySound)],
								stderr=s.STDOUT, stdout=s.PIPE)
					break
	return w.WEECHAT_RC_OK

def fn_command(data, buffer, args):
	global bfList
	global muted
	args = args.split()
	myStatus = "on"
	mySound = ""
	myBuffer = ""
	if len(args)>0:
		try: # set a valid buffer and soundfile
			myArg = args[1]
			myBuffer = w.buffer_search("irc", myArg)
			if myBuffer == "":
				# no buffer with that name, so so assume it's the soundfile
				mySound = myArg
				myBuffer = w.current_buffer()
			else: # we have a buffer, now look for a soundfile
				try:
					mySound = args[2]
				except:
					mySound = ""
		except: # no further arguments, so set valid buffer + sound
			myBuffer = w.current_buffer()
			mySound = settings["psound"]
		myBuffer = w.buffer_get_string(myBuffer, "name")
		if args[0] == "test":
			if settings["psound"] == "":
				errMsg("No sound defined! Please set 'psound'-option!")
			else:
				s.Popen([settings["player"], expanduser(mySound)],
						  stderr=s.STDOUT, stdout=s.PIPE)
		elif args[0] == "list":
			for lEntry in bfList:
				alert("BUFFER: " + lEntry["buffer"] + " | " +
					  "STATUS: " + lEntry["status"] + " | " +
					  "SOUND:  " + lEntry["sound"])
			if len(bfList) == 0:
				errMsg("No buffers configured!")
		elif args[0] == "muteall":
			muted = True
			alert("All buffers are muted")
		elif args[0] == "demuteall":
			muted = False
			alert("Sounds will be played")
		elif args[0] == "add":
			for listIndex in range(len(bfList)):
				if bfList[listIndex]["buffer"] == myBuffer:
					bfList.pop(listIndex)
					break
			bfList += [{"buffer":myBuffer,"status":myStatus,"sound":mySound}]
			if settings["psound"] == "":
				settings["psound"] = mySound
				w.config_set_plugin("psound", mySound)
			w.config_set_plugin("buffers", fn_createBufferString())
		elif args[0] == "remove":
			for listIndex in range(len(bfList)):
				if bfList[listIndex]["buffer"] == myBuffer:
					bfList.pop(listIndex)
					break
			w.config_set_plugin("buffers", fn_createBufferString())
		elif args[0] == "on":
			for listIndex in range(len(bfList)):
				if bfList[listIndex]["buffer"] == myBuffer:
					bfList[listIndex]["status"] = "on"
					w.config_set_plugin("buffers", fn_createBufferString())
					break
		elif args[0] == "off":
			for lIndex in range(len(bfList)):
				if bfList[lIndex]["buffer"] == myBuffer:
					bfList[lIndex]["status"] = "off"
					w.config_set_plugin("buffers", fn_createBufferString())
					break
	return w.WEECHAT_RC_OK

def fn_createBufferString():
	global bfList
	myString = ""
	for lEntry in bfList:
		myString += "{'" + lEntry["buffer"] + "','" + lEntry["status"]
		myString += "','" + lEntry["sound"] + "'}"
	return myString

def alert(myString):
	w.prnt("", myString)
	return

def fn_configchange(data, option, value):
	global settings
	fields = option.split(".")
	myOption = fields[-1]
	try:
		settings[myOption] = value
		#alert("Option {0} has been changed to {1}".format(myOption,
		#			settings[myOption]))
	except KeyError:
		errMsg("There is no option named %s" %myOption)
	return w.WEECHAT_RC_OK

if __name__ == "__main__" and import_ok:
	if w.register(SCR_NAME, SCR_AUTHOR, SCR_VERSION, SCR_LICENSE,
						SCR_DESC, "", ""):
		# synchronize weechat- and scriptsettings
		for option, default_value in settings.items():
			if not w.config_is_set_plugin(option):
				w.config_set_plugin(option, default_value)
			else:
				settings[option] = w.config_get_plugin(option)
				if option == "buffers":	# need to set buffers seperately
					myBuffers = settings[option][2:-2]
					try:
						myBuffers = myBuffers.split("'}{'")
						for tmp in myBuffers:
							myBuffer, myStatus, mySound = tmp.split("','")
							bfList += [{"buffer":myBuffer,"status":myStatus,
										 "sound":mySound}]
					except:
						myBuffers = ""
		w.hook_print("", "", "", 1, "fn_privmsg", "") # catch prvmsg
		w.hook_config("plugins.var.python." + SCR_NAME + ".*",
							"fn_configchange", "") # catch configchanges
		w.hook_command( # help-text
			SCR_COMMAND, SCR_DESC,
			"[test] | [add] [buffer] [sound[] | [remove] buffer | " +
			"[on] [buffer] | [off] [buffer] | [muteall] | " +
			"[demuteall] | [list]",
			"Attention:" +
"""
* in places where you can enter a buffername, its best to use the 'Tab'-key
  to cycle through available buffers. If no buffer is chosen at all or the
  buffer doesn't exist, the current buffer will be used.
* You are free to choose whatever player you want (by entering its' name. I
  urge you to *not* enter any command that could be dangerous, as the
  command will be executed just as if it were entered on the commandline.
  The suggestion for common players would be 'mplayer', 'vlc' or 'ogg123';
  without the quotes of course).
  
Available options are:
- player:  The application used to play a soundfile
- psound:  The soundfile that should be played
- buffers: The saved buffers (it's best to not edit that setting directly.
  Rather choose one of the above mentioned commands).
  
Commands:
- test:      Test soundplaying with current settings
- add:       add/edit a buffer. If no sound is chosen, the standard on is being
             used (if one is set)
- remove:    Remove a buffer.
- off:       Turn off sound for buffer
- on:        Turn on sound for buffer
- muteall:   Turn off sounds for all buffers
- demuteall: Turn on sounds for all buffers that were activated before
- list:      List all saved buffers""",
			"|| add %(buffers_names)"
			"|| remove %(buffers_names)"
			"|| on %(buffers_names)"
			"|| off %(buffers_names)"
			"mute"
			"muteall", "fn_command", ""
			)



#
# Copyright (c) 2009 by Benjamin Neff <info@benjaminneff.ch>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# moc control and now playing script for Weechat.
# (this script requires WeeChat 0.3.0 (or newer) and moc)
#
# History:
#
# 2009-10-26, Benjamin Neff <info@benjaminneff.ch>:
#     version 1.7.3: - Bugfix ( "/me" --> "output_nothing" ) 2
# 2009-10-25, Benjamin Neff <info@benjaminneff.ch>:
#     version 1.7.2: - Fix if moc isn't running
# 2009-10-24, Benjamin Neff <info@benjaminneff.ch>:
#     version 1.7.1: - Bugfix ( "/me" --> "output_nothing" )
# 2009-10-16, Benjamin Neff <info@benjaminneff.ch>:
#     version 1.7: - AvgBitrate bug
#                  - format Time
# 2009-09-15, Benjamin Neff <info@benjaminneff.ch>:
#     version 1.6: - bugfixing ;-)
#                  - hook_config
#                  - remove i|o|ot (color-bug)
# 2009-09-08, Benjamin Neff <info@benjaminneff.ch>:
#     version 1.5: initial release / port to weechat 0.3.0
#

SCRIPT_NAME    = "moc_control"
SCRIPT_AUTHOR  = "SuperTux88 (Benjamin Neff) <info@benjaminneff.ch>"
SCRIPT_VERSION = "1.7.3"
SCRIPT_LICENSE = "GPL2"
SCRIPT_DESC    = "moc control and now playing script for Weechat"

SCRIPT_COMMAND = "moc"

import_ok      = True

try:
    import weechat
except ImportError:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    import_ok = False

try:
    import os
    import subprocess
    import traceback
except ImportError, message:
    print "Missing package(s) for %s: %s" % (SCRIPT_NAME, message)
    import_ok = False

# =================================[ config ]=================================

infobar        = {}
output         = {}

STATUS_NOT_RUNNING = 'NOT_RUNNING'
STATUS_PLAYING     = 'PLAY'
STATUS_PAUSED      = 'PAUSE'
STATUS_STOPPED     = 'STOP'

def load_settings(data, option, value):
    """load all settings"""
    infobar['enabled'] = _load_setting('infobar_enabled', 'off', 'bool')
    infobar['format']  = _load_setting('infobar_format', 'Now Playing: %mocTitle%')
    infobar['update']  = _load_setting('infobar_update', '10', 'int')
    output['format']   = _load_setting('output_format', '/me is listening to %C04%title%%C %artist%%album%::: %C07%file%%C ::: %cTime%/%tTime% @ %bitrate%')
    output['artist']   = _load_setting('output_format_artist', '- %C03%artist%%C ')
    output['album']    = _load_setting('output_format_album', '(%C12%album%%C) ')
    output['nothing']  = _load_setting('output_nothing', '/me is listening to nothing')

    #update config for 1.6 (i|o|ot -> +/me)
    if weechat.config_is_set_plugin('output_type'):
        if weechat.config_get_plugin('output_type') == 'ot':
            weechat.config_set_plugin('output_format', "/me " + output['format'])
            weechat.config_set_plugin('output_nothing', "/me " + output['nothing'])
            output['format'] = "/me " + output['format']
            output['nothing'] = "/me " + output['nothing']
        weechat.config_unset_plugin('output_type')

    return weechat.WEECHAT_RC_OK

def _load_setting(setting, default=None, type=None):
    """load setting or create it"""
    value = weechat.config_get_plugin(setting)
    if value == '' and default != None:
        weechat.config_set_plugin(setting, default)
        value = default

    if type == 'int':
        value = int(value)

    if type == 'bool':
        if value == "on":
            value = True
        elif value == "off":
            value = False
        else:
            weechat.config_set_plugin(setting, default)
            if default == "on":
                value = True
            elif default == "off":
                value = False

    return value

# ================================[ command ]=================================

def moc_command(data, buffer, args):
    """run command"""
    args = args.split(' ')
    if args[0] == '':
        return moc_now_playing(buffer)

    elif args[0] == 'pause' or args[0] == 'pp':
        if _get_status() == STATUS_STOPPED:
            weechat.prnt(buffer, 'moc: Not playing')
            return weechat.WEECHAT_RC_OK
        else:
            _execute_command('mocp -G')
            weechat.prnt(buffer, 'moc: Song paused / Continue playing')
            return weechat.WEECHAT_RC_OK
    elif args[0] == 'play':
        if _get_status() == STATUS_PLAYING:
            weechat.prnt(buffer, 'moc: Already playing')
            return weechat.WEECHAT_RC_OK
        elif _get_status() == STATUS_PAUSED:
            _execute_command('mocp -U')
            weechat.prnt(buffer, 'moc: Continue playing.')
            return weechat.WEECHAT_RC_OK
        else:
            _execute_command('mocp -p')
            weechat.prnt(buffer, 'moc: Started playing.')
            return weechat.WEECHAT_RC_OK
    elif args[0] == 'stop':
        _execute_command('mocp -s')
        weechat.prnt(buffer, 'moc: Stop playing.')
        return weechat.WEECHAT_RC_OK
    elif args[0] == 'prev':
        if _get_status() == STATUS_STOPPED:
            weechat.prnt(buffer, 'moc: Not playing, cannot go to previous song.')
            return weechat.WEECHAT_RC_OK
        else:
            _execute_command('mocp -r')
            weechat.prnt(buffer, 'moc: Playing previous song.')
            return weechat.WEECHAT_RC_OK
    elif args[0] == 'next':
        if _get_status() == STATUS_STOPPED:
            weechat.prnt(buffer, 'moc: Not playing, cannot go to next song.')
            return weechat.WEECHAT_RC_OK
        else:
            _execute_command('mocp -f')
            weechat.prnt(buffer, 'moc: Playing next song.')
            return weechat.WEECHAT_RC_OK
    elif args[0] == 'infobar':
        if infobar['enabled']:
            infobar['enabled'] = False
            weechat.config_set_plugin('infobar_enabled', 'off')
            _remove_infobar()
        else:
            infobar['enabled'] = True
            weechat.config_set_plugin('infobar_enabled', 'on')
            _add_infobar()
        return weechat.WEECHAT_RC_OK
    elif args[0] == 'help':
        weechat.command(buffer, '/help moc')
        return weechat.WEECHAT_RC_OK
    else:
        weechat.prnt(buffer, 'moc: Unknown command %s' % (args[0]))
        return weechat.WEECHAT_RC_OK

# ================================[ infobar ]=================================

def moc_infobar_update(data, buffer, args):
    """Callback for the bar item"""
    if _get_status() == STATUS_NOT_RUNNING:
        return 'moc is not running'
    if _get_status() == STATUS_STOPPED:
        return 'moc is not currently playing'
    else:
        song = _get_song_info()
        return _format_np(infobar['format'], song, 'infobar')
    
def moc_infobar_updater(data,cals):
    """Update the bar item"""
    if infobar['enabled']:
        weechat.bar_item_update('moc_infobar')
    return weechat.WEECHAT_RC_OK

def _add_infobar():
    """add the infobar for moc_control"""
    weechat.bar_new(SCRIPT_NAME, "off", "750", "window", "", "bottom", "horizontal", "vertical", "1", "0", "default", "blue", "cyan", "off", "[moc_infobar]")
    
def _remove_infobar():
    """remove the infobar for moc_control"""
    weechat.bar_remove(weechat.bar_search(SCRIPT_NAME))

# ==============================[ now playing ]===============================

def moc_now_playing(buffer):
    """print now playing"""
    format = ''

    status = _get_status()
    if status == STATUS_STOPPED or status == STATUS_NOT_RUNNING:
        format = output['nothing']
    else:
        song = _get_song_info()
        format = _format_np(output['format'], song, 'chat')

    weechat.command(buffer, format)

    return weechat.WEECHAT_RC_OK

def _format_np(np, song, npType):
    """format the 'now Playing'-String"""
    np = np.replace('%mocTitle%', song['Title'])
    np = np.replace('%title%', song['SongTitle'])

    if npType == 'chat':
        if song['Artist'] != 'unknown':
            np = np.replace('%artist%', output['artist'])
            np = np.replace('%artist%', song['Artist'])
        else:
            np = np.replace('%artist%', '')

        if song['Album'] != 'unknown':
            np = np.replace('%album%', output['album'])
            np = np.replace('%album%', song['Album'])
        else:
            np = np.replace('%album%', '')
    else:
        np = np.replace('%artist%', song['Artist'])
        np = np.replace('%album%', song['Album'])

    np = np.replace('%cTime%', song['CurrentTime'])
    np = np.replace('%cSec%', song['CurrentSec'])
    np = np.replace('%tTime%', song['TotalTime'])
    np = np.replace('%tSec%', song['TotalSec'])
    np = np.replace('%bitrate%', song['Bitrate'])
    np = np.replace('%avgBitrate%', song['AvgBitrate'])
    np = np.replace('%rate%', song['Rate'])
    np = np.replace('%file%', song['File'])
    np = np.replace('%C', chr(3))

    if _get_status() == STATUS_PAUSED:
        np = np + " - [PAUSED]"

    return np

def _get_song_info():
    """Get the song information from moc"""
    song = {}
    song['TotalSec'] = '??'
    song['AvgBitrate'] = '???Kbps'

    info = _execute_command('mocp -i')
    for line in info.split('\n'):
        if line != '':
            index = line.find(': ')
            name = line[:index]
            value = line[index+2:]
            if value == '':
                value = 'unknown'
            song[name] = value.strip()

    if song['File'].find("://") < 0:
        song['File'] = os.path.basename(song['File'])

    if song['TotalSec'] == '??':
        song['TotalTime'] = '?:??'
    else:
        if song['TotalTime'].find("m") > 0:
            song['TotalTime'] = _format_seconds(song['TotalSec'])

    if song['CurrentTime'].find("m") > 0:
        song['CurrentTime'] = _format_seconds(song['CurrentSec'])

    return song

def _format_seconds(s):
    """return the formated time"""
    s = int(s)
    temp = float()
    temp = float(s) / (60 * 60 * 24)
    d = int(temp)
    temp = (temp - d) * 24
    h = int(temp)
    temp = (temp - h) * 60
    m = int(temp)
    temp = (temp - m) * 60
    sec = temp
    if d > 0:
        return "%id %i:%02i:%02i" % (d, h, m, sec)
    else:
        return "%i:%02i:%02i" % (h, m, sec)

def _get_status():
    """return the Status of moc"""
    return _execute_command('mocp -i | grep "State:" | cut -d " " -f 2').strip()

def _execute_command(cmd):
    """execute a command"""
    from subprocess import PIPE
    proc = subprocess.Popen(cmd, shell = True, stderr = PIPE, stdout = PIPE, close_fds = True)
    error = proc.stderr.read()
    if error != '':
        for line in error.split('\n'):
            if line == 'FATAL_ERROR: The server is not running':
                return STATUS_NOT_RUNNING
            
    output = proc.stdout.read()
    proc.wait()
    return output

# ==================================[ main ]==================================

if __name__ == "__main__" and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "moc_unload", ""):
        load_settings('', '', '')
        weechat.hook_config('plugins.var.python.moc_control.*', 'load_settings', '')

        weechat.bar_item_new('moc_infobar', 'moc_infobar_update', '')
        weechat.hook_timer(infobar['update']*1000,0,0,'moc_infobar_updater','')

        if infobar['enabled']:
            _add_infobar()
        weechat.hook_command(
            SCRIPT_COMMAND,
            'Control moc or display now playing information.',
            'play|pause|pp|stop|prev|next|infobar|help',
            'Commands Available\n'
            '  /moc         - Display currently playing song.\n'
            '  /moc play    - Start playing music.\n'
            '  /moc pause   - Toggle between pause/playing.\n'
            '  /moc pp      - Toggle between pause/playing.\n'
            '  /moc stop    - Stop playing music.\n'
            '  /moc prev    - Move to the previous song in the playlist.\n'
            '  /moc next    - Move to the next song in the playlist.\n'
            '  /moc infobar - Toggle the infobar display.\n'
            '\n'
            'Formatting\n'
            '  %mocTitle%   - Replaced with the title from moc.\n'
            '  %artist%     - Replaced with the song artist.\n'
            '  %title%      - Replaced with the song title.\n'
            '  %album%      - Replaced with the song album.\n'
            '  %file%       - Replaced with the filename/url of the song.\n'
            '  %cTime%      - Replaced with how long the song has been playing.\n'
            '  %cSec%       - Replaced with how long the song has been playing (seconcs).\n'
            '  %tTime%      - Replaced with the length of the song.\n'
            '  %tSec%       - Replaced with the length of the song (seconcs).\n'
            '  %bitrate%    - Replaced with the bitrate of the song.\n'
            '  %avgBitrate% - Replaced with the AvgBitrate of the song.\n'
            '  %rate%       - Replaced with the rate of the song.\n'
            '  %C##         - Make ## the number code of the color you want to use. Use %C by itself to end the color.\n'
            '\n'
            'To see all available settings, please check /set *moc_control*\n',
            'play|pause|pp|stop|prev|next|infobar|help',
            'moc_command',
            ''
        )

# ==================================[ end ]===================================
        
def moc_unload():
    """Unload the plugin from weechat"""
    if infobar['enabled']:
        _remove_infobar()
    return weechat.WEECHAT_RC_OK



# HighlightXMPP 0.5 for IRC. Requires WeeChat >= 0.3.0,
# Python >= 2.6, and sleekxmpp.
# Repo: https://github.com/jpeddicord/weechat-highlightxmpp
# 
# Copyright (c) 2009-2015 Jacob Peddicord <jacob@peddicord.net>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#######
#
# You must configure this plugin before using:
#
#   JID messages are sent from:
#     /set plugins.var.python.highlightxmpp.jid someid@jabber.org
#   alternatively, to use a specific resource:
#     /set plugins.var.python.highlightxmpp.jid someid@jabber.org/resource
#
#   Password for the above JID:
#     /set plugins.var.python.highlightxmpp.password abcdef
#
#   JID messages are sent *to* (if not set, defaults to the same jid as above):
#     /set plugins.var.python.highlightxmpp.to myid@jabber.org

import sys
import weechat as w
import sleekxmpp

if sys.version_info < (3, 0):
    from sleekxmpp.util.misc_ops import setdefaultencoding
    setdefaultencoding('utf8')

info = (
    'highlightxmpp',
    'Jacob Peddicord <jacob@peddicord.net>',
    '0.5',
    'GPL3',
    "Relay highlighted & private IRC messages over XMPP (Jabber)",
    '',
    ''
)

settings = {
    'jid': '',
    'password': '',
    'to': '',
}

class SendMsgBot(sleekxmpp.ClientXMPP):
    def __init__(self, jid, password, recipient, message):
        sleekxmpp.ClientXMPP.__init__(self, jid, password)
        self.jid = jid
        self.recipient = recipient
        self.msg = message
        self.add_event_handler("session_start", self.start, threaded=True)
    def start(self, event):
        self.send_presence()
        self.get_roster()
        self.send_message(mto=self.recipient,
                          mbody=self.msg,
                          mtype='chat')
        self.disconnect(wait=True)


def send_xmpp(data, signal, message, trial=1):
    jid = w.config_get_plugin('jid')
    jid_to = w.config_get_plugin('to')
    if not jid_to:
        jid_to = jid
    password = w.config_get_plugin('password')

    xmpp = SendMsgBot(jid, password, jid_to, message)
    if not xmpp.connect():
        w.prnt('', "Unable to connect to XMPP server.")
        return w.WEECHAT_RC_OK
    xmpp.process(block=True)
    return w.WEECHAT_RC_OK


# register with weechat
if w.register(*info):
    # add our settings
    for setting in settings:
        if not w.config_is_set_plugin(setting):
            w.config_set_plugin(setting, settings[setting])
    # and finally our hooks
    w.hook_signal('weechat_highlight', 'send_xmpp', '')
    w.hook_signal('weechat_pv', 'send_xmpp', '')



# -*- coding: utf-8 -*-
#
# Copyright (c) 2009 by xt <xt@bash.no>
# Borrowed parts from pagetitle.py by xororand
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
#
# If someone posts an URL in a configured channel
# this script will post back title

# Explanation about ignores:
#   * plugins.var.python.announce_url_title.ignore_buffers:
#   Comma separated list of patterns for define ignores.
#   URLs from channels where its name matches any of these patterns will be ignored.
#   Wildcards '*', '?' and char groups [..] can be used.
#   An ignore exception can be added by prefixing '!' in the pattern.
#
#       Example:
#       *ubuntu*,!#ubuntu-offtopic
#       any urls from a 'ubuntu' channel will be ignored,
#       except from #ubuntu-offtopic
#
#   * plugins.var.python.announce_url_title.url_ignore
#     simply does partial match, so specifying 'google' will ignore every url with the word google in it
#
#
# History:
#
# 2014-05-10, Sébastien Helleu <flashcode@flashtux.org>
#   version 18: change hook_print callback argument type of displayed/highlight
#               (WeeChat >= 1.0)
# 2013-11-07, excalibr
#   version 17: add more characters to exclude in escaping (this fix problem with youtube urls)
# 2012-11-15, xt
#   version 16: improve escaping
# 2011-09-04, Deltafire
#   version 15: fix remote execution exploit due to unescaped ' character in urls;
#               small bug fix for version 14 changes
# 2011-08-23, Deltafire
#   version 14: ignore filtered lines
# 2011-03-11, Sébastien Helleu <flashcode@flashtux.org>
#   version 13: get python 2.x binary for hook_process (fix problem when python 3.x is default python version)
# 2010-12-10, xt
#   version 12: add better ignores (code based on m4v inotify.py)
# 2010-11-02, xt
#   version 11: add prefix
# 2010-11-01, xt
#   version 10: add ignored buffers feature
# 2010-10-29, add ignore buffers feature
#   version 0.9: WeeChat user-agent option
# 2010-10-11, xt
#   version 0.8: support multiple concurrent url lookups
# 2010-10-11, xt
#   version 0.7: do not trigger on notices
# 2010-08-25, xt
#   version 0.6: notice some buffers instead of msg
# 2009-12-08, Chaz6
#   version 0.5: only announce for specified channels
# 2009-12-08, Chaz6 <chaz@chaz6.com>
#   version 0.4: add global option
# 2009-12-08, xt
#   version 0.3: option for public announcing or not
# 2009-12-07, xt <xt@bash.no>
#   version 0.2: don't renannounce same urls for a time
#                add optional prefix and suffix
# 2009-12-02, xt
#   version 0.1: initial

import weechat
w = weechat
import re
import htmllib
from time import time as now
from fnmatch import fnmatch
from urllib import quote

SCRIPT_NAME    = "announce_url_title"
SCRIPT_AUTHOR  = "xt <xt@bash.no>"
SCRIPT_VERSION = "18"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Announce URL titles to channel or locally"

settings = {
    "buffers"        : 'freenode.#testing,',     # comma separated list of buffers
    "buffers_notice" : 'freenode.#testing,',     # comma separated list of buffers
    'ignore_buffers' : 'grep,',     # comma separated list of buffers to be ignored by this module
    'title_max_length': '80',
    'url_ignore'     : '', # comma separated list of strings in url to ignore
    'reannounce_wait': '5', # 5 minutes delay
    'prefix':   '',
    'suffix':   '',
    'announce_public': 'off', # print it or msg the buffer
    'global': 'off', # whether to enable for all buffers
    'user_agent': 'WeeChat/%(version)s (http://www.weechat.org)', # user-agent format string
    'global_prefix':'url', # Prefix for when not public announcement
}


octet = r'(?:2(?:[0-4]\d|5[0-5])|1\d\d|\d{1,2})'
ipAddr = r'%s(?:\,.%s){3}' % (octet, octet)
# Base domain regex off RFC 1034 and 1738
label = r'[0-9a-z][-0-9a-z]*[0-9a-z]?'
domain = r'%s(?:\.%s)*\.[a-z][-0-9a-z]*[a-z]?' % (label, label)
urlRe = re.compile(r'(\w+://(?:%s|%s)(?::\d+)?(?:/[^\])>\s]*)?)' % (domain, ipAddr), re.I)

buffer_name = ''

urls = {}
script_nick = 'url'
def say(s, buffer=''):
    """normal msg"""
    weechat.prnt(buffer, '%s\t%s' %(script_nick, s))

def unescape(s):
    """Unescape HTML entities"""
    p = htmllib.HTMLParser(None)
    p.save_bgn()
    p.feed(s)
    return p.save_end()

def url_print_cb(data, buffer, time, tags, displayed, highlight, prefix, message):
    global buffer_name, urls, ignore_buffers

    # Do not trigger on filtered lines and notices
    if not int(displayed) or prefix == '--':
        return w.WEECHAT_RC_OK

    msg_buffer_name = w.buffer_get_string(buffer, "name")
    # Skip ignored buffers

    #if msg_buffer_name in w.config_get_plugin('ignore_buffers').split(','):
    #    return w.WEECHAT_RC_OK
    if msg_buffer_name in ignore_buffers:
        return w.WEECHAT_RC_OK

    found = False
    notice = False
    if w.config_get_plugin('global') == 'on':
        found = True
        buffer_name = msg_buffer_name
    else:
        for active_buffer in w.config_get_plugin('buffers').split(','):
            if active_buffer.lower() == msg_buffer_name.lower():
                found = True
                buffer_name = msg_buffer_name
                break
        for active_buffer in w.config_get_plugin('buffers_notice').split(','):
            if active_buffer.lower() == msg_buffer_name.lower():
                found = True
                buffer_name = msg_buffer_name
                break

    if not found:
        return w.WEECHAT_RC_OK

    ignorelist = w.config_get_plugin('url_ignore').split(',')
    for url in urlRe.findall(message):

        url = quote(url, "%/:=&?~#+!$,;@()*[]") # Escape URL
        ignore = False
        for ignore_part in ignorelist:
            if ignore_part.strip():
                if ignore_part in url:
                    ignore = True
                    w.prnt('', '%s: Found %s in URL: %s, ignoring.' %(SCRIPT_NAME, ignore_part, url))
                    break

        if ignore:
            continue

        if url in urls:
            continue
        else:
            urls[url] = {}
    url_process_launcher()

    return w.WEECHAT_RC_OK

def url_process_launcher():
    ''' Iterate found urls, fetch title if hasn't been launched '''
    global urls

    user_agent = w.config_get_plugin('user_agent') % {'version': w.info_get("version", "")}
    for url, url_d in urls.items():
        if not url_d: # empty dict means not launched
            url_d['launched'] = now()

            # Read 8192
            python2_bin = w.info_get("python2_bin", "") or "python"
            cmd = python2_bin + " -c \"import urllib2; opener = urllib2.build_opener();"
            cmd += "opener.addheaders = [('User-agent','%s')];" % user_agent
            cmd += "print opener.open('%s').read(8192)\"" % url

            url_d['stdout'] = ''
            url_d['url_hook_process'] = w.hook_process(cmd, 30 * 1000, "url_process_cb", "")

    return w.WEECHAT_RC_OK

def url_process_cb(data, command, rc, stdout, stderr):
    """ Callback parsing html for title """

    global buffer_name, urls

    url = command.split("'")[-2]
    if stdout != "":
        urls[url]['stdout'] += stdout
    if int(rc) >= 0:

        head = re.sub("[\r\n\t ]"," ", urls[url]['stdout'])
        title = re.search('(?i)\<title\>(.*?)\</title\>', head)
        if title:
            title = unescape(title.group(1))

            max_len = int(w.config_get_plugin('title_max_length'))
            if len(title) > max_len:
                title = "%s [...]" % title[0:max_len]

            splits = buffer_name.split('.') #FIXME bad code
            server = splits[0]
            buffer = '.'.join(splits[1:])
            output = w.config_get_plugin('prefix') + title + w.config_get_plugin('suffix')
            announce_public = w.config_get_plugin('announce_public')
            if announce_public == 'on':
                found = False
                for active_buffer in w.config_get_plugin('buffers').split(','):
                    if active_buffer.lower() == buffer_name.lower():
                        w.command('', '/msg -server %s %s %s' %(server, buffer, output))
                        found = True
                for active_buffer in w.config_get_plugin('buffers_notice').split(','):
                    if active_buffer.lower() == buffer_name.lower():
                        w.command('', '/notice -server %s %s %s' %(server, buffer, output))
                        found = True
                if found == False:
                    say(output,w.buffer_search('', buffer_name))
            else:
                say(output,w.buffer_search('', buffer_name))
        urls[url]['stdout'] = ''

    return w.WEECHAT_RC_OK

def purge_cb(*args):
    ''' Purge the url list on configured intervals '''

    global urls

    t_now = now()
    for url, url_d in urls.items():
        if (t_now - url_d['launched']) > \
            int(w.config_get_plugin('reannounce_wait'))*60:
                del urls[url]

    return w.WEECHAT_RC_OK

class Ignores(object):
    def __init__(self, ignore_type):
        self.ignore_type = ignore_type
        self.ignores = []
        self.exceptions = []
        self._get_ignores()

    def _get_ignores(self):
        assert self.ignore_type is not None
        ignores = weechat.config_get_plugin(self.ignore_type).split(',')
        ignores = [ s.lower() for s in ignores if s ]
        self.ignores = [ s for s in ignores if s[0] != '!' ]
        self.exceptions = [ s[1:] for s in ignores if s[0] == '!' ]

    def __contains__(self, s):
        s = s.lower()
        for p in self.ignores:
            if fnmatch(s, p):
                for e in self.exceptions:
                    if fnmatch(s, e):
                        return False
                return True
        return False

def ignore_update(*args):
    ignore_buffers._get_ignores()
    return w.WEECHAT_RC_OK


if __name__ == "__main__":
    if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                        SCRIPT_DESC, "", ""):

        # Set default settings
        for option, default_value in settings.iteritems():
            if not w.config_is_set_plugin(option):
                w.config_set_plugin(option, default_value)
        ignore_buffers = Ignores('ignore_buffers')

        w.hook_print("", "", "://", 1, "url_print_cb", "")
        w.hook_timer(\
            int(w.config_get_plugin('reannounce_wait')) * 1000 * 60,
            0,
            0,
            "purge_cb",
            '')
        weechat.hook_config('plugins.var.python.%s.ignore_buffers' %SCRIPT_NAME, 'ignore_update', '')
    color_chat_delimiters = weechat.color('chat_delimiters')
    color_chat_nick       = weechat.color('chat_nick')
    color_reset           = weechat.color('reset')
    color_chat_buffer     = weechat.color('chat_buffer')
    # pretty printing
    script_nick = '%s[%s%s%s]%s' %(color_chat_delimiters,
                                   color_chat_nick,
                                   w.config_get_plugin('global_prefix'),
                                   color_chat_delimiters,
                                   color_reset)
